/**
 * math.js
 * https://github.com/josdejong/mathjs
 *
 * Math.js is an extensive math library for JavaScript and Node.js,
 * It features real and complex numbers, units, matrices, a large set of
 * mathematical functions, and a flexible expression parser.
 *
 * @version 0.5.0
 * @date    2013-04-06
 *
 * @license
 * Copyright (C) 2013 Jos de Jong <wjosdejong@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy
 * of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
(function(){function e(e){return e instanceof Boolean||"boolean"==typeof e}function n(e,t){if(this.constructor!=n)throw new SyntaxError("Complex constructor must be called with the new operator");if(null!=e&&!v(e)||null!=t&&!v(t))throw new TypeError("Two numbers or a single string expected in Complex constructor");this.re=e||0,this.im=t||0}function t(e){if(this.constructor!=t)throw new SyntaxError("Matrix constructor must be called with the new operator");if(e instanceof t||e instanceof g)this._data=e.toArray();else if(e instanceof Array)this._data=e;else{if(null!=e)throw new TypeError("Unsupported type of data ("+at.typeof(e)+")");this._data=[]}this._size=it.size(this._data)}function r(e,n){if(!v(e)||!d(e))throw new TypeError("Index must be an integer (value: "+e+")");if(1>e)throw new RangeError("Index out of range ("+e+" < 1)");if(n&&e>n)throw new RangeError("Index out of range ("+e+" > "+n+")")}function a(e,n){return r(n,e.length),e[n-1]}function i(e,n){return n.forEach(function(n){e=a(e,n)}),jn(e)}function o(e,n){var t=n[0];return t.map?t.map(function(n){return a(e,n)}):[a(e,t)]}function s(e,n){var t=n[0],r=n[1];if(t.map)return r.map?t.map(function(n){var t=a(e,n);return r.map(function(e){return a(t,e)})}):t.map(function(n){return[a(a(e,n),r)]});if(r.map){var i=a(e,t);return[r.map(function(e){return a(i,e)})]}return[[a(a(e,t),r)]]}function f(e,n,t){var r=t==n.length-1,i=n[t],o=function(i){var o=a(e,i);return r?o:f(o,n,t+1)};return i.map?i.map(o):[o(i)]}function u(e,n,t){if(r(n),t instanceof Array)throw new TypeError("Dimension mismatch, value expected instead of array");e[n-1]=t}function c(e,n,t,a){var i=!1;t.length>n.length&&(i=!0);for(var o=0;t.length>o;o++){var s=t[o];r(s),(null==n[o]||s>n[o])&&(n[o]=s,i=!0)}i&&it.resize(e,n,0);var f=n.length;t.forEach(function(n,t){f-1>t?e=e[n-1]:e[n-1]=a})}function l(e,n,t,a){var i=t[0];r(i),i>n[0]&&it.resize(e,[i],0),e[i-1]=a}function h(e,n,t,a){var i=t[0],o=t[1];r(i),r(o);var s=!1;i>(n[0]||0)&&(n[0]=i,s=!0),o>(n[1]||0)&&(n[1]=o,s=!0),s&&it.resize(e,n,0),e[i-1][o-1]=a}function m(e,n,t,r,a){var i=r==t.length-1,o=t[r],s=function(o,s){if(i)u(e,o,a[s]),e.length>(n[r]||0)&&(n[r]=e.length);else{var f=e[o-1];f instanceof Array||(e[o-1]=f=[f],e.length>(n[r]||0)&&(n[r]=e.length)),m(f,n,t,r+1,a[s])}};if(o.map){var f=o.size&&o.size()||o.length;if(f!=a.length)throw new RangeError("Dimensions mismatch ("+f+" != "+a.length+")");o.map(s)}else s(o,0)}function p(e){for(var n=0,t=e.length;t>n;n++){var r=e[n];r instanceof Array?p(r):void 0==r&&(e[n]=0)}}function v(e){return e instanceof Number||"number"==typeof e}function d(e){return e==Math.round(e)}function g(e,n,t){if(this.constructor!=g)throw new SyntaxError("Range constructor must be called with the new operator");if(null!=e&&!v(e))throw new TypeError("Parameter start must be a number");if(null!=t&&!v(t))throw new TypeError("Parameter end must be a number");if(null!=n&&!v(n))throw new TypeError("Parameter step must be a number");this.start=null!=e?e:0,this.end=null!=t?t:0,this.step=null!=n?n:1}function y(e){return e instanceof String||"string"==typeof e}function x(e,n){if(this.constructor!=x)throw Error("Unit constructor must be called with the new operator");if(null!=e&&!v(e))throw Error("First parameter in Unit constructor must be a number");if(null!=n&&!y(n))throw Error("Second parameter in Unit constructor must be a string");if(this.value=1,this.unit=x.UNIT_NONE,this.prefix=x.PREFIX_NONE,this.hasUnit=!1,this.hasValue=!1,this.fixPrefix=!1,null!=n){for(var t=x.UNITS,r=!1,a=0,i=t.length;i>a;a++){var o=t[a];if(x.endsWith(n,o.name)){var s=n.length-o.name.length,f=n.substring(0,s),u=o.prefixes[f];if(void 0!==u){this.unit=o,this.prefix=u,this.hasUnit=!0,r=!0;break}}}if(!r)throw Error('String "'+n+'" is no unit')}null!=e?(this.value=this._normalize(e),this.hasValue=!0):this.value=this._normalize(1)}function w(e,n){var t=void 0;if(2==arguments.length){var r=Xn(n);t="Function "+e+" does not support a parameter of type "+r}else if(arguments.length>2){for(var a=[],i=1;arguments.length>i;i++)a.push(Xn(arguments[i]));t="Function "+e+" does not support a parameters of type "+a.join(", ")}else t="Unsupported parameter in function "+e;return new TypeError(t)}function E(e,n,t,r){var a="Wrong number of arguments in function "+e+" ("+n+" provided, "+t+(void 0!=r?"-"+r:"")+" expected)";return new SyntaxError(a)}function N(e){if(1!=arguments.length)throw E("abs",arguments.length,1);if(v(e))return Math.abs(e);if(e instanceof n)return Math.sqrt(e.re*e.re+e.im*e.im);if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,N);if(e.valueOf()!==e)return N(e.valueOf());throw w("abs",e)}function b(e,r){if(2!=arguments.length)throw E("add",arguments.length,2);if(v(e)){if(v(r))return e+r;if(r instanceof n)return new n(e+r.re,r.im)}else if(e instanceof n){if(v(r))return new n(e.re+r,e.im);if(r instanceof n)return new n(e.re+r.re,e.im+r.im)}else if(e instanceof x&&r instanceof x){if(!e.equalBase(r))throw Error("Units do not match");if(!e.hasValue)throw Error("Unit on left hand side of operator + has no value");if(!r.hasValue)throw Error("Unit on right hand side of operator + has no value");var a=e.clone();return a.value+=r.value,a.fixPrefix=!1,a}if(y(e)||y(r))return e+r;if(e instanceof Array||e instanceof t||e instanceof g||r instanceof Array||r instanceof t||r instanceof g)return it.map2(e,r,b);if(e.valueOf()!==e)return b(e.valueOf(),r.valueOf());throw w("add",e,r)}function O(e){if(1!=arguments.length)throw E("ceil",arguments.length,1);if(v(e))return Math.ceil(e);if(e instanceof n)return new n(Math.ceil(e.re),Math.ceil(e.im));if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,O);if(e.valueOf()!==e)return O(e.valueOf());throw w("ceil",e)}function M(e){if(1!=arguments.length)throw E("cube",arguments.length,1);if(v(e))return e*e*e;if(e instanceof n)return P(P(e,e),e);if(e instanceof Array||e instanceof t||e instanceof g)return P(P(e,e),e);if(e.valueOf()!==e)return M(e.valueOf());throw w("cube",e)}function A(e,r){if(2!=arguments.length)throw E("divide",arguments.length,2);if(v(e)){if(v(r))return e/r;if(r instanceof n)return S(new n(e,0),r)}if(e instanceof n){if(v(r))return S(e,new n(r,0));if(r instanceof n)return S(e,r)}if(e instanceof x&&v(r)){var a=e.clone();return a.value/=r,a}if(e instanceof Array||e instanceof t)return r instanceof Array||r instanceof t?at.multiply(e,at.inv(r)):it.map2(e,r,A);if(r instanceof Array||r instanceof t)return at.multiply(e,at.inv(r));if(e.valueOf()!==e||r.valueOf()!==r)return A(e.valueOf(),r.valueOf());throw w("divide",e,r)}function S(e,t){var r=t.re*t.re+t.im*t.im;return new n((e.re*t.re+e.im*t.im)/r,(e.im*t.re-e.re*t.im)/r)}function T(e,r){if(2!=arguments.length)throw E("equal",arguments.length,2);if(v(e)){if(v(r))return e==r;if(r instanceof n)return e==r.re&&0==r.im}if(e instanceof n){if(v(r))return e.re==r&&0==e.im;if(r instanceof n)return e.re==r.re&&e.im==r.im}if(e instanceof x&&r instanceof x){if(!e.equalBase(r))throw Error("Cannot compare units with different base");return e.value==r.value}if(y(e)||y(r))return e==r;if(e instanceof Array||e instanceof t||e instanceof g||r instanceof Array||r instanceof t||r instanceof g)return it.map2(e,r,T);if(e.valueOf()!==e||r.valueOf()!==r)return T(e.valueOf(),r.valueOf());throw w("equal",e,r)}function z(e){if(1!=arguments.length)throw E("exp",arguments.length,1);if(v(e))return Math.exp(e);if(e instanceof n){var r=Math.exp(e.re);return new n(r*Math.cos(e.im),r*Math.sin(e.im))}if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,z);if(e.valueOf()!==e)return z(e.valueOf());throw w("exp",e)}function q(e){if(1!=arguments.length)throw E("fix",arguments.length,1);if(v(e))return e>0?Math.floor(e):Math.ceil(e);if(e instanceof n)return new n(e.re>0?Math.floor(e.re):Math.ceil(e.re),e.im>0?Math.floor(e.im):Math.ceil(e.im));if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,q);if(e.valueOf()!==e)return q(e.valueOf());throw w("fix",e)}function U(e){if(1!=arguments.length)throw E("floor",arguments.length,1);if(v(e))return Math.floor(e);if(e instanceof n)return new n(Math.floor(e.re),Math.floor(e.im));if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,U);if(e.valueOf()!==e)return U(e.valueOf());throw w("floor",e)}function R(e,r){if(2!=arguments.length)throw E("larger",arguments.length,2);if(v(e)){if(v(r))return e>r;if(r instanceof n)return e>N(r)}if(e instanceof n){if(v(r))return N(e)>r;if(r instanceof n)return N(e)>N(r)}if(e instanceof x&&r instanceof x){if(!e.equalBase(r))throw Error("Cannot compare units with different base");return e.value>r.value}if(y(e)||y(r))return e>r;if(e instanceof Array||e instanceof t||e instanceof g||r instanceof Array||r instanceof t||r instanceof g)return it.map2(e,r,R);if(e.valueOf()!==e||r.valueOf()!==r)return R(e.valueOf(),r.valueOf());throw w("larger",e,r)}function L(e,r){if(2!=arguments.length)throw E("largereq",arguments.length,2);if(v(e)){if(v(r))return e>=r;if(r instanceof n)return e>=N(r)}if(e instanceof n){if(v(r))return N(e)>=r;if(r instanceof n)return N(e)>=N(r)}if(e instanceof x&&r instanceof x){if(!e.equalBase(r))throw Error("Cannot compare units with different base");return e.value>=r.value}if(y(e)||y(r))return e>=r;if(e instanceof Array||e instanceof t||e instanceof g||r instanceof Array||r instanceof t||r instanceof g)return it.map2(e,r,L);if(e.valueOf()!==e||r.valueOf()!==r)return L(e.valueOf(),r.valueOf());throw w("largereq",e,r)}function C(e,r){if(1!=arguments.length&&2!=arguments.length)throw E("log",arguments.length,1,2);if(void 0!==r)return A(C(e),C(r));if(v(e))return e>=0?Math.log(e):C(new n(e,0));if(e instanceof n)return new n(Math.log(Math.sqrt(e.re*e.re+e.im*e.im)),Math.atan2(e.im,e.re));if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,C);if(e.valueOf()!==e||r.valueOf()!==r)return C(e.valueOf(),r.valueOf());throw w("log",e,r)}function _(e){if(1!=arguments.length)throw E("log10",arguments.length,1);if(v(e))return e>=0?Math.log(e)/Math.LN10:_(new n(e,0));if(e instanceof n)return new n(Math.log(Math.sqrt(e.re*e.re+e.im*e.im))/Math.LN10,Math.atan2(e.im,e.re)/Math.LN10);if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,_);if(e.valueOf()!==e)return _(e.valueOf());throw w("log10",e)}function I(e,r){if(2!=arguments.length)throw E("mod",arguments.length,2);if(v(e)){if(v(r))return e%r;if(r instanceof n&&0==r.im)return e%r.re}else if(e instanceof n&&0==e.im){if(v(r))return e.re%r;if(r instanceof n&&0==r.im)return e.re%r.re}if(e instanceof Array||e instanceof t||e instanceof g||r instanceof Array||r instanceof t||r instanceof g)return it.map2(e,r,I);if(e.valueOf()!==e||r.valueOf()!==r)return I(e.valueOf(),r.valueOf());throw w("mod",e,r)}function P(e,r){if(2!=arguments.length)throw E("multiply",arguments.length,2);if(v(e)){if(v(r))return e*r;if(r instanceof n)return B(new n(e,0),r);if(r instanceof x)return o=r.clone(),o.value*=e,o}else if(e instanceof n){if(v(r))return B(e,new n(r,0));if(r instanceof n)return B(e,r)}else if(e instanceof x){if(v(r))return o=e.clone(),o.value*=r,o}else{if(e instanceof Array){if(r instanceof Array){var a=it.size(e),i=it.size(r);if(2!=a.length)throw Error("Can only multiply a 2 dimensional matrix (A has "+a.length+" dimensions)");if(2!=i.length)throw Error("Can only multiply a 2 dimensional matrix (B has "+i.length+" dimensions)");if(a[1]!=i[0])throw new RangeError("Dimensions mismatch in multiplication. Columns of A must match rows of B (A is "+a[0]+"x"+a[1]+", B is "+i[0]+"x"+i[1]+", "+i[1]+" != "+i[0]+")");for(var o=[],s=a[0],f=i[1],u=a[1],c=0;s>c;c++){o[c]=[];for(var l=0;f>l;l++){for(var h=null,m=0;u>m;m++){var p=P(e[c][m],r[m][l]);h=null==h?p:b(h,p)}o[c][l]=h}}return o}return r instanceof t?new t(P(e.valueOf(),r.valueOf())):it.map2(e,r,P)}if(e instanceof t)return new t(P(e.valueOf(),r.valueOf()))}if(r instanceof Array)return it.map2(e,r,P);if(r instanceof t)return new t(P(e.valueOf(),r.valueOf()));if(e.valueOf()!==e||r.valueOf()!==r)return P(e.valueOf(),r.valueOf());throw w("multiply",e,r)}function B(e,t){return new n(e.re*t.re-e.im*t.im,e.re*t.im+e.im*t.re)}function k(e,r){if(2!=arguments.length)throw E("pow",arguments.length,2);if(v(e)){if(v(r))return d(r)||e>=0?Math.pow(e,r):G(new n(e,0),new n(r,0));if(r instanceof n)return G(new n(e,0),r)}else if(e instanceof n){if(v(r))return G(e,new n(r,0));if(r instanceof n)return G(e,r)}else{if(e instanceof Array){if(!v(r)||!d(r)||0>r)throw new TypeError("For A^b, b must be a positive integer (value is "+r+")");var a=it.size(e);if(2!=a.length)throw Error("For A^b, A must be 2 dimensional (A has "+a.length+" dimensions)");if(a[0]!=a[1])throw Error("For A^b, A must be square (size is "+a[0]+"x"+a[1]+")");if(0==r)return identity(a[0]);for(var i=e,o=1;r>o;o++)i=P(e,i);return i}if(e instanceof t)return new t(k(e.valueOf(),r))}if(e.valueOf()!==e||r.valueOf()!==r)return k(e.valueOf(),r.valueOf());throw w("pow",e,r)}function G(e,n){var t=C(e),r=P(t,n);return z(r)}function D(e,r){if(1!=arguments.length&&2!=arguments.length)throw E("round",arguments.length,1,2);if(void 0==r){if(v(e))return Math.round(e);if(e instanceof n)return new n(Math.round(e.re),Math.round(e.im));if((e instanceof Array||e instanceof t||e instanceof g)&&it.map(e,D),e.valueOf()!==e)return D(e.valueOf());throw w("round",e)}if(!v(r))throw new TypeError("Number of digits in function round must be an integer");if(r!==Math.round(r))throw new TypeError("Number of digits in function round must be integer");if(0>r||r>9)throw Error("Number of digits in function round must be in te range of 0-9");if(v(e))return j(e,r);if(e instanceof n)return new n(j(e.re,r),j(e.im,r));if(e instanceof Array||e instanceof t||e instanceof g||r instanceof Array||r instanceof t||r instanceof g)return it.map2(e,r,D);if(e.valueOf()!==e||r.valueOf()!==r)return R(e.valueOf(),r.valueOf());throw w("round",e,r)}function j(e,n){var t=Math.pow(10,void 0!=n?n:at.options.precision);return Math.round(e*t)/t}function V(e){if(1!=arguments.length)throw E("sign",arguments.length,1);if(v(e)){var r;return r=e>0?1:0>e?-1:0}if(e instanceof n){var a=Math.sqrt(e.re*e.re+e.im*e.im);return new n(e.re/a,e.im/a)}if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,r);if(e.valueOf()!==e)return r(e.valueOf());throw w("sign",e)}function F(e,r){if(2!=arguments.length)throw E("smaller",arguments.length,2);if(v(e)){if(v(r))return r>e;if(r instanceof n)return N(r)>e}if(e instanceof n){if(v(r))return r>N(e);if(r instanceof n)return N(e)<N(r)}if(e instanceof x&&r instanceof x){if(!e.equalBase(r))throw Error("Cannot compare units with different base");return e.value<r.value}if(y(e)||y(r))return r>e;if(e instanceof Array||e instanceof t||e instanceof g||r instanceof Array||r instanceof t||r instanceof g)return it.map2(e,r,F);if(e.valueOf()!==e||r.valueOf()!==r)return F(e.valueOf(),r.valueOf());throw w("smaller",e,r)}function H(e,r){if(2!=arguments.length)throw E("smallereq",arguments.length,2);if(v(e)){if(v(r))return r>=e;if(r instanceof n)return N(r)>=e}if(e instanceof n){if(v(r))return r>=N(e);if(r instanceof n)return N(e)<=N(r)}if(e instanceof x&&r instanceof x){if(!e.equalBase(r))throw Error("Cannot compare units with different base");return e.value<=r.value}if(y(e)||y(r))return r>=e;if(e instanceof Array||e instanceof t||e instanceof g||r instanceof Array||r instanceof t||r instanceof g)return it.map2(e,r,H);if(e.valueOf()!==e||r.valueOf()!==r)return H(e.valueOf(),r.valueOf());throw w("smallereq",e,r)}function Y(e){if(1!=arguments.length)throw E("sqrt",arguments.length,1);if(v(e))return e>=0?Math.sqrt(e):Y(new n(e,0));if(e instanceof n){var r=Math.sqrt(e.re*e.re+e.im*e.im);return e.im>=0?new n(.5*Math.sqrt(2*(r+e.re)),.5*Math.sqrt(2*(r-e.re))):new n(.5*Math.sqrt(2*(r+e.re)),-.5*Math.sqrt(2*(r-e.re)))}if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,Y);if(e.valueOf()!==e)return Y(e.valueOf());throw w("sqrt",e)}function W(e){if(1!=arguments.length)throw E("square",arguments.length,1);if(v(e))return e*e;if(e instanceof n)return P(e,e);if(e instanceof Array||e instanceof t||e instanceof g)return P(e,e);if(e.valueOf()!==e)return W(e.valueOf());throw w("square",e)}function X(e,r){if(2!=arguments.length)throw E("subtract",arguments.length,2);if(v(e)){if(v(r))return e-r;if(r instanceof n)return new n(e-r.re,r.im)}else if(e instanceof n){if(v(r))return new n(e.re-r,e.im);if(r instanceof n)return new n(e.re-r.re,e.im-r.im)}else if(e instanceof x&&r instanceof x){if(!e.equalBase(r))throw Error("Units do not match");if(!e.hasValue)throw Error("Unit on left hand side of operator - has no value");if(!r.hasValue)throw Error("Unit on right hand side of operator - has no value");var a=e.clone();return a.value-=r.value,a.fixPrefix=!1,a}if(e instanceof Array||e instanceof t||e instanceof g||r instanceof Array||r instanceof t||r instanceof g)return it.map2(e,r,X);if(e.valueOf()!==e||r.valueOf()!==r)return X(e.valueOf(),r.valueOf());throw w("subtract",e,r)}function Z(e){if(1!=arguments.length)throw E("unaryminus",arguments.length,1);if(v(e))return-e;if(e instanceof n)return new n(-e.re,-e.im);if(e instanceof x){var r=e.clone();return r.value=-e.value,r}if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,Z);if(e.valueOf()!==e)return Z(e.valueOf());throw w("unaryminus",e)}function K(e,r){if(2!=arguments.length)throw E("unequal",arguments.length,2);if(v(e)){if(v(r))return e==r;if(r instanceof n)return e==r.re&&0==r.im}if(e instanceof n){if(v(r))return e.re==r&&0==e.im;if(r instanceof n)return e.re==r.re&&e.im==r.im}if(e instanceof x&&r instanceof x){if(!e.equalBase(r))throw Error("Cannot compare units with different base");return e.value==r.value}if(y(e)||y(r))return e==r;if(e instanceof Array||e instanceof t||e instanceof g||r instanceof Array||r instanceof t||r instanceof g)return it.map2(e,r,K);if(e.valueOf()!==e||r.valueOf()!==r)return K(e.valueOf(),r.valueOf());throw w("unequal",e,r)}function Q(e){if(1!=arguments.length)throw E("arg",arguments.length,1);if(v(e))return Math.atan2(0,e);if(e instanceof n)return Math.atan2(e.im,e.re);if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,Q);if(e.valueOf()!==e)return Q(e.valueOf());throw w("arg",e)}function J(e){if(1!=arguments.length)throw E("conj",arguments.length,1);if(v(e))return e;if(e instanceof n)return new n(e.re,-e.im);if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,J);if(e.valueOf()!==e)return J(e.valueOf());throw w("conj",e)}function $(e){if(1!=arguments.length)throw E("im",arguments.length,1);if(v(e))return 0;if(e instanceof n)return e.im;if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,$);if(e.valueOf()!==e)return $(e.valueOf());throw w("im",e)}function en(e){if(1!=arguments.length)throw E("re",arguments.length,1);if(v(e))return e;if(e instanceof n)return e.re;if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,en);if(e.valueOf()!==e)return en(e.valueOf());throw w("re",e)}function nn(){switch(arguments.length){case 0:return new n(0,0);case 1:var e=arguments[0];if(!y(e))throw new TypeError("Two numbers or a single string expected in function complex");var t=n.parse(e);if(t)return t;throw new SyntaxError('String "'+e+'" is no valid complex number');case 2:return new n(arguments[0],arguments[1]);default:throw E("complex",arguments.length,0,2)}}function tn(e){if(arguments.length>1)throw E("matrix",arguments.length,0,1);return new t(e)}function rn(){return new at.expr.Parser}function an(e){switch(arguments.length){case 1:if(!y(e))throw new TypeError("Two or three numbers or a single string expected in function range");var n=g.parse(e);if(n)return n;throw new SyntaxError('String "'+n+'" is no valid range');case 2:return new g(arguments[0],null,arguments[1]);case 3:return new g(arguments[0],arguments[1],arguments[2]);default:throw E("range",arguments.length,2,3)}}function on(){switch(arguments.length){case 1:var e=arguments[0];if(!y(e))throw new TypeError("A string or a number and string expected in function unit");if(x.isUnit(e))return new x(null,e);var n=x.parse(e);if(n)return n;throw new SyntaxError('String "'+e+'" is no valid unit');case 2:return new x(arguments[0],arguments[1]);default:throw E("unit",arguments.length,1,2)}}function sn(){return new at.expr.Workspace}function fn(){var e,n,r=arguments.length,a=-1,i=!1,o=[];for(e=0;r>e;e++){var s=arguments[e];if(s instanceof t&&(i=!0),e==r-1&&v(s)){if(n=a,a=s,!d(a)||1>a)throw new TypeError("Dimension number must be a positive integer (dim = "+a+")");if(e>0&&a>n)throw new RangeError("Dimension out of range ("+a+" > "+n+")")}else{if(!(s instanceof Array||s instanceof t))throw w("concat",s);var f=at.clone(s.valueOf()),u=at.size(s);if(o[e]=f,n=a,a=u.length,e>0&&a!=n)throw new RangeError("Dimension mismatch ("+n+" != "+a+")")}}if(0==o.length)throw new SyntaxError("At least one matrix expected");for(var c=o.shift();o.length;)c=un(c,o.shift(),a-1,0);return i?new t(c):c}function un(e,n,t,r){if(t>r){if(e.length!=n.length)throw Error("Dimensions mismatch ("+e.length+" != "+n.length+")");for(var a=[],i=0;e.length>i;i++)a[i]=un(e[i],n[i],t,r+1);return a}return e.concat(n)}function cn(e){if(1!=arguments.length)throw E("det",arguments.length,1);var n=at.size(e);switch(n.length){case 0:return at.clone(e);case 1:if(1==n[0])return at.clone(e.valueOf()[0]);throw new RangeError("Matrix must be square (size: "+at.format(n)+")");case 2:var t=n[0],r=n[1];if(t==r)return ln(e.valueOf(),t,r);throw new RangeError("Matrix must be square (size: "+at.format(n)+")");default:throw new RangeError("Matrix must be two dimensional (size: "+at.format(n)+")")}}function ln(e,n,t){var r=at.multiply,a=at.subtract;if(1==n)return e[0][0];if(2==n)return a(r(e[0][0],e[1][1]),r(e[1][0],e[0][1]));for(var i=0,o=0;t>o;o++){var s=hn(e,n,t,0,o);i+=r(r((o+1)%2+(o+1)%2-1,e[0][o]),ln(s,n-1,t-1))}return i}function hn(e,n,t,r,a){for(var i,o=[],s=0;n>s;s++)if(s!=r){i=o[s-(s>r)]=[];for(var f=0;t>f;f++)f!=a&&(i[f-(f>a)]=e[s][f])}return o}function mn(e,n){var r,a,i,o;if(1!=arguments.length&&2!=arguments.length)throw E("diag",arguments.length,1,2);if(n){if(!v(n)||!d(n))throw new TypeError("Second parameter in function diag must be an integer")}else n=0;var s=n>0?n:0,f=0>n?-n:0;e instanceof t||e instanceof g||(e=new t(e));var u;switch(e.isVector()?(e=e.toVector(),u=[e.length]):u=e.size(),u.length){case 1:a=e.valueOf();var c=new t;for(c.resize([a.length+f,a.length+s]),r=c.valueOf(),o=a.length,i=0;o>i;i++)r[i+f][i+s]=jn(a[i]);return c;case 2:for(a=[],r=e.valueOf(),o=Math.min(u[0]-f,u[1]-s),i=0;o>i;i++)a[i]=jn(r[i+f][i+s]);return new t(a);default:throw new RangeError("Matrix for function diag must be 2 dimensional")}}function pn(){var e=it.argsToArray(arguments);if(0==e.length)e=[1,1];else if(1==e.length)e[1]=e[0];else if(e.length>2)throw E("eye",num,0,2);var n=e[0],r=e[1];if(!v(n)||!d(n)||1>n)throw Error("Parameters in function eye must be positive integers");if(r&&(!v(r)||!d(r)||1>r))throw Error("Parameters in function eye must be positive integers");var a=new t;a.resize(e);for(var i=at.min(e),o=a.valueOf(),s=0;i>s;s++)o[s][s]=1;return a}function vn(e){if(1!=arguments.length)throw E("inv",arguments.length,1);var n=at.size(e);switch(n.length){case 0:return at.divide(1,e);case 1:if(1==n[0])return e instanceof t?new t([at.divide(1,e.valueOf()[0])]):[at.divide(1,e[0])];throw new RangeError("Matrix must be square (size: "+at.format(n)+")");case 2:var r=n[0],a=n[1];if(r==a)return e instanceof t?new t(dn(e.valueOf(),r,a)):dn(e,r,a);throw new RangeError("Matrix must be square (size: "+at.format(n)+")");default:throw new RangeError("Matrix must be two dimensional (size: "+at.format(n)+")")}}function dn(e,n,t){var r,a,i,o,s,f=at.add,u=at.unaryminus,c=at.multiply,l=at.divide;if(1==n){if(o=e[0][0],0==o)throw Error("Cannot calculate inverse, determinant is zero");return[[at.divide(1,o)]]}if(2==n){var h=at.det(e);if(0==h)throw Error("Cannot calculate inverse, determinant is zero");return[[l(e[1][1],h),l(u(e[0][1]),h)],[l(u(e[1][0]),h),l(e[0][0],h)]]}var m=e.concat();for(r=0;n>r;r++)m[r]=m[r].concat();for(var p=at.eye(n).valueOf(),v=0;t>v;v++){for(r=v;n>r&&0==m[r][v];)r++;if(r==n||0==m[r][v])throw Error("Cannot calculate inverse, determinant is zero");r!=v&&(s=m[v],m[v]=m[r],m[r]=s,s=p[v],p[v]=p[r],p[r]=s);var d=m[v],g=p[v];for(r=0;n>r;r++){var y=m[r],x=p[r];if(r!=v){if(0!=y[v]){for(i=l(u(y[v]),d[v]),a=v;t>a;a++)y[a]=f(y[a],c(i,d[a]));for(a=0;t>a;a++)x[a]=f(x[a],c(i,g[a]))}}else{for(i=d[v],a=v;t>a;a++)y[a]=l(y[a],i);for(a=0;t>a;a++)x[a]=l(x[a],i)}}}return p}function gn(){var e=it.argsToArray(arguments);0==e.length?e=[1,1]:1==e.length&&(e[1]=e[0]);var n=new t,r=1;return n.resize(e,r),n}function yn(e){if(1!=arguments.length)throw E("size",arguments.length,1);if(v(e)||e instanceof n||e instanceof x||null==e)return[];if(y(e))return[e.length];if(e instanceof Array)return it.size(e);if(e instanceof t||e instanceof g)return e.size();if(e.valueOf()!==e)return yn(e.valueOf());throw w("size",e)}function xn(e){if(1!=arguments.length)throw E("squeeze",arguments.length,1);return e instanceof t||e instanceof g?wn(e.toArray()):e instanceof Array?wn(jn(e)):jn(e)}function wn(e){if(1==e.length)return wn(e[0]);for(var n=0,t=e.length;t>n;n++){var r=e[n];r instanceof Array&&(e[n]=wn(r))}return e}function En(e){if(1!=arguments.length)throw E("transpose",arguments.length,1);var n=at.size(e);switch(n.length){case 0:return at.clone(e);case 1:return at.clone(e);case 2:for(var r,a=n[1],i=n[0],o=e instanceof t,s=e.valueOf(),f=[],u=at.clone,c=0;a>c;c++){r=f[c]=[];for(var l=0;i>l;l++)r[l]=u(s[l][c])}return 0==i&&(f[0]=[]),o?new t(f):f;default:throw new RangeError("Matrix must be two dimensional (size: "+at.format(n)+")")}}function Nn(){var e=it.argsToArray(arguments);0==e.length?e=[1,1]:1==e.length&&(e[1]=e[0]);var n=new t;return n.resize(e),n}function bn(e){if(1!=arguments.length)throw E("factorial",arguments.length,1);if(v(e)){if(!d(e))throw new TypeError("Function factorial can only handle integer values");var n=e,r=n;for(n--;n>1;)r*=n,n--;return 0==r&&(r=1),r}if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,bn);if(e.valueOf()!==e)return bn(e.valueOf());throw w("factorial",e)}function On(){if(0!=arguments.length)throw E("random",arguments.length,0);return Math.random()}function Mn(e){if(0==arguments.length)throw Error("Function max requires one or more parameters (0 provided)");if(e instanceof Array||e instanceof t||e instanceof g){if(arguments.length>1)throw Error("Wrong number of parameters (1 matrix or multiple scalars expected)");var n=at.size(e);if(1==n.length){if(0==e.length)throw Error("Cannot calculate max of an empty vector");return An(e.valueOf())}if(2==n.length){if(0==n[0]||0==n[1])throw Error("Cannot calculate max of an empty matrix");if(e instanceof Array)return Sn(e,n[0],n[1]);if(e instanceof t||e instanceof g)return new t(Sn(e.valueOf(),n[0],n[1]));throw w("max",e)}throw new RangeError("Cannot calculate max for multi dimensional matrix")}return An(arguments)}function An(e){for(var n=at.larger,t=e[0],r=1,a=e.length;a>r;r++){var i=e[r];n(i,t)&&(t=i)}return t}function Sn(e,n,t){for(var r=at.larger,a=[],i=0;t>i;i++){for(var o=e[0][i],s=1;n>s;s++){var f=e[s][i];r(f,o)&&(o=f)}a[i]=o}return a}function Tn(e){if(0==arguments.length)throw Error("Function min requires one or more parameters (0 provided)");if(e instanceof Array||e instanceof t||e instanceof g){if(arguments.length>1)throw Error("Wrong number of parameters (1 matrix or multiple scalars expected)");var n=at.size(e);if(1==n.length){if(0==e.length)throw Error("Cannot calculate min of an empty vector");return zn(e.valueOf())}if(2==n.length){if(0==n[0]||0==n[1])throw Error("Cannot calculate min of an empty matrix");if(e instanceof Array)return qn(e,n[0],n[1]);if(e instanceof t||e instanceof g)return new t(qn(e.valueOf(),n[0],n[1]));throw w("min",e)}throw new RangeError("Cannot calculate min for multi dimensional matrix")}return zn(arguments)}function zn(e){for(var n=at.smaller,t=e[0],r=1,a=e.length;a>r;r++){var i=e[r];n(i,t)&&(t=i)}return t}function qn(e,n,t){for(var r=at.smaller,a=[],i=0;t>i;i++){for(var o=e[0][i],s=1;n>s;s++){var f=e[s][i];r(f,o)&&(o=f)}a[i]=o}return a}function Un(e){if(1!=arguments.length)throw E("acos",arguments.length,1);if(v(e))return e>=-1&&1>=e?Math.acos(e):Un(new n(e,0));if(e instanceof n){var r=new n(e.im*e.im-e.re*e.re+1,-2*e.re*e.im),a=Y(r),i=new n(a.re-e.im,a.im+e.re),o=C(i);return new n(1.5707963267948966-o.im,o.re)}if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,Un);if(e.valueOf()!==e)return Un(e.valueOf());throw w("acos",e)}function Rn(e){if(1!=arguments.length)throw E("asin",arguments.length,1);if(v(e))return e>=-1&&1>=e?Math.asin(e):Rn(new n(e,0));if(e instanceof n){var r=e.re,a=e.im,i=new n(a*a-r*r+1,-2*r*a),o=Y(i),s=new n(o.re-a,o.im+r),f=C(s);return new n(f.im,-f.re)}if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,Rn);if(e.valueOf()!==e)return Rn(e.valueOf());throw w("asin",e)}function Ln(e){if(1!=arguments.length)throw E("atan",arguments.length,1);if(v(e))return Math.atan(e);if(e instanceof n){var r=e.re,a=e.im,i=r*r+(1-a)*(1-a),o=new n((1-a*a-r*r)/i,-2*r/i),s=C(o);return new n(-.5*s.im,.5*s.re)}if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,Ln);if(e.valueOf()!==e)return Ln(e.valueOf());throw w("atan",e)}function Cn(e,r){if(2!=arguments.length)throw E("atan2",arguments.length,2);if(v(e)){if(v(r))return Math.atan2(e,r);if(r instanceof n)return Math.atan2(e,r.re)}else if(e instanceof n){if(v(r))return Math.atan2(e.re,r);if(r instanceof n)return Math.atan2(e.re,r.re)}if(e instanceof Array||e instanceof t||e instanceof g||r instanceof Array||r instanceof t||r instanceof g)return it.map2(e,r,Cn);if(r.valueOf()!==r||e.valueOf()!==e)return Cn(e.valueOf(),r.valueOf());throw w("atan2",e,r)}function _n(e){if(1!=arguments.length)throw E("cos",arguments.length,1);if(v(e))return Math.cos(e);if(e instanceof n)return new n(.5*Math.cos(e.re)*(Math.exp(-e.im)+Math.exp(e.im)),.5*Math.sin(e.re)*(Math.exp(-e.im)-Math.exp(e.im)));if(e instanceof x){if(!e.hasBase(x.BASE_UNITS.ANGLE))throw new TypeError("Unit in function cos is no angle");return Math.cos(e.value)}if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,_n);if(e.valueOf()!==e)return _n(e.valueOf());throw w("cos",e)}function In(e){if(1!=arguments.length)throw E("cot",arguments.length,1);if(v(e))return 1/Math.tan(e);if(e instanceof n){var r=Math.exp(-4*e.im)-2*Math.exp(-2*e.im)*Math.cos(2*e.re)+1;return new n(2*Math.exp(-2*e.im)*Math.sin(2*e.re)/r,(Math.exp(-4*e.im)-1)/r)}if(e instanceof x){if(!e.hasBase(x.BASE_UNITS.ANGLE))throw new TypeError("Unit in function cot is no angle");return 1/Math.tan(e.value)}if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,In);if(e.valueOf()!==e)return In(e.valueOf());throw w("cot",e)}function Pn(e){if(1!=arguments.length)throw E("csc",arguments.length,1);if(v(e))return 1/Math.sin(e);if(e instanceof n){var r=.25*(Math.exp(-2*e.im)+Math.exp(2*e.im))-.5*Math.cos(2*e.re);return new n(.5*Math.sin(e.re)*(Math.exp(-e.im)+Math.exp(e.im))/r,.5*Math.cos(e.re)*(Math.exp(-e.im)-Math.exp(e.im))/r)}if(e instanceof x){if(!e.hasBase(x.BASE_UNITS.ANGLE))throw new TypeError("Unit in function csc is no angle");return 1/Math.sin(e.value)}if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,Pn);if(e.valueOf()!==e)return Pn(e.valueOf());throw w("csc",e)}function Bn(e){if(1!=arguments.length)throw E("sec",arguments.length,1);if(v(e))return 1/Math.cos(e);if(e instanceof n){var r=.25*(Math.exp(-2*e.im)+Math.exp(2*e.im))+.5*Math.cos(2*e.re);return new n(.5*Math.cos(e.re)*(Math.exp(-e.im)+Math.exp(e.im))/r,.5*Math.sin(e.re)*(Math.exp(e.im)-Math.exp(-e.im))/r)}if(e instanceof x){if(!e.hasBase(x.BASE_UNITS.ANGLE))throw new TypeError("Unit in function sec is no angle");return 1/Math.cos(e.value)}if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,Bn);if(e.valueOf()!==e)return Bn(e.valueOf());throw w("sec",e)}function kn(e){if(1!=arguments.length)throw E("sin",arguments.length,1);
if(v(e))return Math.sin(e);if(e instanceof n)return new n(.5*Math.sin(e.re)*(Math.exp(-e.im)+Math.exp(e.im)),.5*Math.cos(e.re)*(Math.exp(e.im)-Math.exp(-e.im)));if(e instanceof x){if(!e.hasBase(x.BASE_UNITS.ANGLE))throw new TypeError("Unit in function cos is no angle");return Math.sin(e.value)}if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,kn);if(e.valueOf()!==e)return kn(e.valueOf());throw w("sin",e)}function Gn(e){if(1!=arguments.length)throw E("tan",arguments.length,1);if(v(e))return Math.tan(e);if(e instanceof n){var r=Math.exp(-4*e.im)+2*Math.exp(-2*e.im)*Math.cos(2*e.re)+1;return new n(2*Math.exp(-2*e.im)*Math.sin(2*e.re)/r,(1-Math.exp(-4*e.im))/r)}if(e instanceof x){if(!e.hasBase(x.BASE_UNITS.ANGLE))throw new TypeError("Unit in function tan is no angle");return Math.tan(e.value)}if(e instanceof Array||e instanceof t||e instanceof g)return it.map(e,Gn);if(e.valueOf()!==e)return Gn(e.valueOf());throw w("tan",e)}function Dn(e,n){if(2!=arguments.length)throw E("in",arguments.length,2);if(e instanceof x&&n instanceof x){if(!e.equalBase(n))throw Error("Units do not match");if(n.hasValue)throw Error("Cannot convert to a unit with a value");if(!n.hasUnit)throw Error("Unit expected on the right hand side of function in");var r=n.clone();return r.value=e.value,r.fixPrefix=!0,r}if(e instanceof Array||e instanceof t||e instanceof g||n instanceof Array||n instanceof t||n instanceof g)return it.map2(e,n,Dn);if(e.valueOf()!==e)return at.in(e.valueOf());throw w("in",e)}function jn(n){if(1!=arguments.length)throw E("clone",arguments.length,1);if(null==n)return n;if("function"==typeof n.clone)return n.clone();if(v(n)||y(n)||e(n))return n;if(n instanceof Array)return n.map(function(e){return jn(e)});if(n instanceof Object)return it.mapObject(n,jn);throw w("clone",n)}function Vn(e,n){var t=arguments.length;if(1!=t&&2!=t)throw E("format",t,1,2);if(1==t){var r=arguments[0];return v(r)?it.formatNumber(r):r instanceof Array?it.formatArray(r):y(r)?'"'+r+'"':r instanceof Object?""+r:r+""}if(!y(e))throw new TypeError("String expected as first parameter in function format");if(!(n instanceof Object))throw new TypeError("Object expected as first parameter in function format");return e.replace(/\$([\w\.]+)/g,function(e,t){for(var r=t.split("."),a=n[r.shift()];r.length&&void 0!=a;){var i=r.shift();a=i?a[i]:a+"."}return void 0!=a?a:e})}function Fn(e){if(1!=arguments.length)throw E("help",arguments.length,1);if(void 0!=e){if(e.doc)return Hn(e.doc);if(e.constructor.doc)return Hn(e.constructor.doc);if(y(e)){var n=at[e];if(n&&n.doc)return Hn(n.doc)}}return e instanceof Object&&e.name?'No documentation found on subject "'+e.name+'"':e instanceof Object&&e.constructor.name?'No documentation found on subject "'+e.constructor.name+'"':'No documentation found on subject "'+e+'"'}function Hn(e){var n="";if(e.name&&(n+="NAME\n"+e.name+"\n\n"),e.category&&(n+="CATEGORY\n"+e.category+"\n\n"),e.syntax&&(n+="SYNTAX\n"+e.syntax.join("\n")+"\n\n"),e.examples){var t=at.parser();n+="EXAMPLES\n";for(var r=0;e.examples.length>r;r++){var a,i=e.examples[r];try{a=t.eval(i)}catch(o){a=o}n+=i+"\n",n+="    "+at.format(a)+"\n"}n+="\n"}return e.seealso&&(n+="SEE ALSO\n"+e.seealso.join(", ")+"\n"),n}function Yn(e,n){var t;if(y(e)){if("undefined"==typeof require)throw Error("Cannot load file: require not available.");var r=require(e);Yn(r)}else if(Wn(e)){if(t=e.name,!t)throw Error("Cannot import an unnamed function");(n||void 0===at[t])&&(at[t]=e)}else if(e instanceof Object)for(t in e)if(e.hasOwnProperty(t)){var a=e[t];Wn(a)?(n||void 0===at[t])&&(at[t]=a):Yn(a)}}function Wn(e){return"function"==typeof e||v(e)||y(e)||e instanceof n||e instanceof x}function Xn(e){if(1!=arguments.length)throw E("typeof",arguments.length,1);var n=typeof e;if("object"==n){if(null==e)return"null";if(e.constructor){for(var t in at)if(at.hasOwnProperty(t)&&e.constructor==at[t])return t.toLowerCase();if(e.constructor.name)return e.constructor.name.toLowerCase()}}return n}function Zn(){}function Kn(e,n,t){this.name=e,this.fn=n,this.params=t}function Qn(e){this.value=e}function Jn(e){this.nodes=e||[]}function $n(){this.params=[],this.visible=[]}function et(e,n,t,r){this.name=e,this.params=n,this.expr=t,this.result=r}function nt(e,n){this.object=e,this.params=n}function tt(e,n,t,r,a){this.name=e,this.variables=t,this.values=[];for(var i=0,o=this.variables.length;o>i;i++)this.values[i]=function(){var e=function(){return e.value};return e.value=void 0,e}();this.def=this.createFunction(e,n,t,r),this.result=a}function rt(e,n){throw Error('Constructor "'+e+'" has been replaced by '+'constructor method "'+n+'" in math.js v0.5.0')}var at={type:{},expr:{node:{}},options:{precision:10}};"undefined"!=typeof module&&module.exports!==void 0&&(module.exports=at),"undefined"!=typeof exports&&(exports=at),"undefined"!=typeof require&&"undefined"!=typeof define&&define(function(){return at}),"undefined"!=typeof window&&(window.math=at);var it=function(){function e(n){if(n instanceof Array){var t=n.length;if(t){var r=e(n[0]);return 0==r[0]?[0].concat(r):[t].concat(r)}return[t]}return[]}function n(e,t,r){var a,i=e.length;if(i!=t[r])throw new RangeError("Dimension mismatch ("+i+" != "+t[r]+")");if(t.length-1>r){var o=r+1;for(a=0;i>a;a++){var s=e[a];if(!(s instanceof Array))throw new RangeError("Dimension mismatch ("+(t.length-1)+" < "+t.length+")");n(e[a],t,o)}}else for(a=0;i>a;a++)if(e[a]instanceof Array)throw new RangeError("Dimension mismatch ("+(t.length+1)+" > "+t.length+")")}function r(e,n,t){if(n.length-1>t){var a=e[0];if(1!=e.length||!(a instanceof Array))throw new RangeError("Dimension mismatch ("+e.length+" > 0)");r(a,n,t+1)}else if(e.length)throw new RangeError("Dimension mismatch ("+e.length+" > 0)")}function a(e,n,t,r){if(!(e instanceof Array))throw new TypeError("Array expected");var i=e.length,o=n[t];if(i!=o){if(o>e.length)for(var s=e.length;o>s;s++)e[s]=r?jn(r):0;else e.length=n[t];i=e.length}if(n.length-1>t){var f=t+1;for(s=0;i>s;s++)u=e[s],u instanceof Array||(u=[u],e[s]=u),a(u,n,f,r)}else for(s=0;i>s;s++){for(var u=e[s];u instanceof Array;)u=u[0];e[s]=u}}var i={};return i.formatNumber=function(e,n){if(1/0===e)return"Infinity";if(e===-1/0)return"-Infinity";if(0/0===e)return"NaN";var t=Math.abs(e);if(t>1e-4&&1e6>t||0==t)return j(e,n)+"";var r=Math.round(Math.log(t)/Math.LN10),a=e/Math.pow(10,r);return j(a,n)+"E"+r},i.formatArray=function(e){if(e instanceof Array){for(var n="[",t=e.length,r=0;t>r;r++)0!=r&&(n+=", "),n+=i.formatArray(e[r]);return n+="]"}return at.format(e)},i.formatArray2d=function(e){var n="[",t=i.size(e);if(2!=t.length)throw new RangeError("Array must be two dimensional (size: "+i.formatArray(t)+")");for(var r=t[0],a=t[1],o=0;r>o;o++){0!=o&&(n+="; ");for(var s=e[o],f=0;a>f;f++){0!=f&&(n+=", ");var u=s[f];void 0!=u&&(n+=Vn(u))}}return n+="]"},i.argsToArray=function(e){var n;if(0==e.length)n=[];else if(1==e.length)n=e[0],n instanceof t&&(n=n.toVector()),n instanceof g&&(n=n.valueOf()),n instanceof Array||(n=[n]);else{n=[];for(var r=0;e.length>r;r++)n[r]=e[r]}return n},i.randomUUID=function(){var e=function(){return Math.floor(65536*Math.random()).toString(16)};return e()+e()+"-"+e()+"-"+e()+"-"+e()+"-"+e()+e()+e()},i.map=function(e,n){if(e instanceof Array||e instanceof t||e instanceof g)return e.map(function(e){return n(e)});throw new TypeError("Array expected")},i.map2=function(e,n,r){var a,o,s;if(e instanceof t||n instanceof t)return new t(i.map2(e.valueOf(),n.valueOf(),r));if(e instanceof g||n instanceof g)return new t(i.map2(e.valueOf(),n.valueOf(),r));if(e instanceof Array)if(n instanceof Array){if(e.length!=n.length)throw new RangeError("Dimension mismatch ("+e.length+" != "+n.length+")");for(a=[],o=e.length,s=0;o>s;s++)a[s]=r(e[s],n[s])}else for(a=[],o=e.length,s=0;o>s;s++)a[s]=r(e[s],n);else if(n instanceof Array)for(a=[],o=n.length,s=0;o>s;s++)a[s]=r(e,n[s]);else a=r(e,n);return a},i.forEach=function(e,n){if(e instanceof Array)e.forEach(n);else for(var t in e)e.hasOwnProperty(t)&&n(e[t],t,e)},i.mapObject=function(e,n){var t={};for(var r in e)e.hasOwnProperty(r)&&(t[r]=n(e[r]));return t},i.deepEqual=function(e,n){var t,r,a;if(e instanceof Array){if(!(n instanceof Array))return!1;for(r=0,a=e.length;a>r;r++)if(!i.deepEqual(e[r],n[r]))return!1;return!0}if(e instanceof Object){if(n instanceof Array||!(n instanceof Object))return!1;for(t in e)if(e.hasOwnProperty(t)&&!i.deepEqual(e[t],n[t]))return!1;for(t in n)if(n.hasOwnProperty(t)&&!i.deepEqual(e[t],n[t]))return!1;return!0}return e.valueOf()==n.valueOf()},i.size=function(n){var t=e(n);return i.validate(n,t),t},i.validate=function(e,t){var a=0==t.length;if(a){if(e instanceof Array)throw new RangeError("Dimension mismatch ("+e.length+" != 0)")}else{var o=-1!=t.indexOf(0);o?(t.forEach(function(e){if(0!=e)throw new RangeError("Invalid size, all dimensions must be either zero or non-zero (size: "+i.formatArray(t)+")")}),r(e,t,0)):n(e,t,0)}},i.resize=function(e,n,t){if(!(n instanceof Array))throw new TypeError("Size must be an array (size is "+at.typeof(n)+")");n.forEach(function(e){if(!v(e)||!d(e)||0>e)throw new TypeError("Invalid size, must contain positive integers (size: "+i.formatArray(n)+")")});var r=-1!=n.indexOf(0);r&&n.forEach(function(e){if(0!=e)throw new RangeError("Invalid size, all dimensions must be either zero or non-zero (size: "+i.formatArray(n)+")")}),a(e,n,0,t)},Array.prototype.indexOf||(Array.prototype.indexOf=function(e){for(var n=0;this.length>n;n++)if(this[n]==e)return n;return-1}),Array.prototype.forEach||(Array.prototype.forEach=function(e,n){for(var t=0,r=this.length;r>t;++t)e.call(n||this,this[t],t,this)}),Array.prototype.map||(Array.prototype.map=function(e,n){var t,r,a;if(null==this)throw new TypeError(" this is null or not defined");var i=Object(this),o=i.length>>>0;if("function"!=typeof e)throw new TypeError(e+" is not a function");for(n&&(t=n),r=Array(o),a=0;o>a;){var s,f;a in i&&(s=i[a],f=e.call(t,s,a,i),r[a]=f),a++}return r}),Array.prototype.every||(Array.prototype.every=function(e){"use strict";if(null==this)throw new TypeError;var n=Object(this),t=n.length>>>0;if("function"!=typeof e)throw new TypeError;for(var r=arguments[1],a=0;t>a;a++)if(a in n&&!e.call(r,n[a],a,n))return!1;return!0}),Array.prototype.some||(Array.prototype.some=function(e){"use strict";if(null==this)throw new TypeError;var n=Object(this),t=n.length>>>0;if("function"!=typeof e)throw new TypeError;for(var r=arguments[1],a=0;t>a;a++)if(a in n&&e.call(r,n[a],a,n))return!0;return!1}),i}();at.type.Complex=n,function(){function e(){for(;" "==c||"	"==c;)a()}function t(e){return e>="0"&&"9">=e||"."==e}function r(e){return e>="0"&&"9">=e}function a(){u++,c=f[u]}function i(e){u=e,c=f[u]}function o(){var e="",n=u;if("+"==c?a():"-"==c&&(e+=c,a()),!t(c))return i(n),null;for(;t(c);)e+=c,a();if("E"==c||"e"==c){if(e+=c,a(),("+"==c||"-"==c)&&(e+=c,a()),!r(c))return i(n),null;for(;r(c);)e+=c,a()}return e}function s(){var e=f[u+1];if("I"==c||"i"==c)return a(),"1";if(!("+"!=c&&"-"!=c||"I"!=e&&"i"!=e)){var n="+"==c?"1":"-1";return a(),a(),n}return null}var f,u,c;n.parse=function(t){if(f=t,u=-1,c="",!y(f))return null;a(),e();var r=o();if(r){if("I"==c||"i"==c)return a(),e(),c?null:new n(0,Number(r));e();var i=c;if("+"!=i&&"-"!=i)return e(),c?null:new n(Number(r),0);a(),e();var l=o();if(l){if("I"!=c&&"i"!=c)return null;a()}else if(l=s(),!l)return null;return"-"==i&&(l="-"==l[0]?"+"+l.substring(1):"-"+l),a(),e(),c?null:new n(Number(r),Number(l))}return(r=s())?(e(),c?null:new n(0,Number(r))):null}}(),n.prototype.clone=function(){return new n(this.re,this.im)},n.prototype.toString=function(){var e="",n=it.formatNumber(this.re),t=it.formatNumber(this.im);return e=0==this.im?n:0==this.re?1==this.im?"i":-1==this.im?"-i":t+"i":this.im>0?1==this.im?n+" + i":n+" + "+t+"i":-1==this.im?n+" - i":n+" - "+it.formatNumber(Math.abs(this.im))+"i"},n.doc={name:"Complex",category:"type",syntax:["a + bi","a + b * i"],description:"A complex value a + bi, where a is the real part and b is the complex part, and i is the imaginary number defined as sqrt(-1).",examples:["2 + 3i","sqrt(-4)","(1.2 -5i) * 2"],seealso:["abs","arg","conj","im","re"]},at.type.Matrix=t,t.prototype.get=function(e){var n;if(e instanceof t)n=e.isVector(),e=e.valueOf();else{if(!(e instanceof Array))throw new TypeError("Unsupported type of index "+at.typeof(e));n=!e.some(function(e){return e.forEach})}if(e.length!=this._size.length)throw new RangeError("Dimension mismatch ("+e.length+" != "+this._size.length+")");if(n)switch(e.length){case 1:return a(this._data,e[0]);case 2:return a(a(this._data,e[0]),e[1]);default:return i(this._data,e)}else switch(e.length){case 1:return new t(o(this._data,e));case 2:return new t(s(this._data,e));default:return new t(f(this._data,e,0))}},t.prototype.set=function(e,n){var r;if(e instanceof t)r=e.isVector(),e=e.valueOf();else{if(!(e instanceof Array))throw new TypeError("Unsupported type of index "+at.typeof(e));r=!e.some(function(e){return e.forEach})}if((n instanceof t||n instanceof g)&&(n=n.valueOf()),e.length<this._size.length)throw new RangeError("Dimension mismatch ("+e.length+" != "+this._size.length+")");if(r){if(0!=at.size(n).length)throw new TypeError("Scalar value expected");switch(e.length){case 1:l(this._data,this._size,e,n);break;case 2:h(this._data,this._size,e,n);break;default:c(this._data,this._size,e,n)}}else{var a=this._size.concat();m(this._data,a,e,0,n),it.deepEqual(this._size,a)||(p(this._data),this.resize(a))}return this},t.prototype.resize=function(e,n){it.resize(this._data,e,n),this._size=jn(e)},t.prototype.clone=function(){var e=new t;return e._data=jn(this._data),e._size=jn(this._size),e},t.prototype.size=function(){return this._size},t.prototype.map=function(e){var n=this,r=new t,a=[],i=function(t,r){return t instanceof Array?t.map(function(e,n){return a[r]=n+1,i(e,r+1)}):e(t,a,n)};return r._data=i(this._data,0),r._size=jn(this._size),r},t.prototype.forEach=function(e){var n=this,t=[],r=function(a,i){a instanceof Array?a.forEach(function(e,n){t[i]=n+1,r(e,i+1)}):e(a,t,n)};r(this._data,0)},t.prototype.toScalar=function(){for(var e=this._data;e instanceof Array&&1==e.length;)e=e[0];return e instanceof Array?null:jn(e)},t.prototype.isScalar=function(){return this._size.every(function(e){return 1>=e})},t.prototype.toVector=function(){var e=0,n=void 0,t=[];if(this._size.forEach(function(r,a){r>1&&(e++,n=a),t[a]=0}),0==e){var r=this.toScalar();return r?[r]:[]}if(1==e){var a=[],i=function(e){e instanceof Array?e.forEach(i):a.push(e)};return i(this._data),a}return null},t.prototype.isVector=function(){var e=0;return this._size.forEach(function(n){n>1&&e++}),1>=e},t.prototype.toArray=function(){return jn(this._data)},t.prototype.valueOf=function(){return this._data},t.prototype.toString=function(){return it.formatArray(this._data)},at.type.Range=g,g.parse=function(e){if(!y(e))return null;var n=e.split(":"),t=n.map(function(e){return Number(e)}),r=t.some(function(e){return isNaN(e)});if(r)return null;switch(t.length){case 2:return new g(t[0],1,t[1]);case 3:return new g(t[0],t[1],t[2]);default:return null}},g.prototype.clone=function(){return new g(this.start,this.step,this.end)},g.prototype.size=function(){var e=0,n=Number(this.start),t=Number(this.step),r=Number(this.end),a=r-n;return V(t)==V(a)?e=Math.floor(a/t)+1:0==a&&(e=1),isNaN(e)&&(e=0),[e]},g.prototype.forEach=function(e){var n=Number(this.start),t=Number(this.step),r=Number(this.end),a=0;if(t>0)for(;r>=n;)e(n,a,this),n+=t,a++;else if(0>t)for(;n>=r;)e(n,a,this),n+=t,a++},g.prototype.map=function(e){var n=[];return this.forEach(function(t,r,a){n[r]=e(t,r,a)}),n},g.prototype.toMatrix=function(){return new t(this.toArray())},g.prototype.toArray=function(){var e=[];return this.forEach(function(n,t){e[t]=n}),e},g.prototype.toVector=g.prototype.toArray,g.prototype.isVector=function(){return!0},g.prototype.toScalar=function(){var e=this.toArray();return 1==e.length?e[0]:null},g.prototype.isScalar=function(){return 1==this.size()[0]},g.prototype.valueOf=function(){return this.toArray()},g.prototype.toString=function(){var e=Vn(Number(this.start));return 1!=this.step&&(e+=":"+Vn(Number(this.step))),e+=":"+Vn(Number(this.end))},at.type.Unit=x,function(){function e(){for(;" "==u||"	"==u;)r()}function n(e){return e>="0"&&"9">=e||"."==e}function t(e){return e>="0"&&"9">=e}function r(){f++,u=s[f]}function a(e){f=e,u=s[f]}function i(){var e="",i=f;if("+"==u?r():"-"==u&&(e+=u,r()),!n(u))return a(i),null;for(;n(u);)e+=u,r();if("E"==u||"e"==u){if(e+=u,r(),("+"==u||"-"==u)&&(e+=u,r()),!t(u))return a(i),null;for(;t(u);)e+=u,r()}return e}function o(){var n="";for(e();u&&" "!=u&&"	"!=u;)n+=u,r();return n||null}var s,f,u;x.parse=function(n){if(s=n,f=-1,u="",!y(s))return null;r(),e();var t,a=i();return a?(t=o(),r(),e(),u?null:a&&t?new x(Number(a),t):null):(t=o(),r(),e(),u?null:new x(null,t))}}(),x.prototype.clone=function(){var e=new x;for(var n in this)this.hasOwnProperty(n)&&(e[n]=this[n]);return e},x.endsWith=function(e,n){var t=e.length-n.length,r=e.length;return e.substring(t,r)===n},x.prototype._normalize=function(e){return(e+this.unit.offset)*this.unit.value*this.prefix.value},x.prototype._unnormalize=function(e,n){return void 0===n?e/this.unit.value/this.prefix.value-this.unit.offset:e/this.unit.value/n-this.unit.offset},x.isUnit=function(e){for(var n=x.UNITS,t=n.length,r=0;t>r;r++){var a=n[r];if(x.endsWith(e,a.name)){var i=e.length-a.name.length;if(0==i)return!0;var o=e.substring(0,i),s=a.prefixes[o];if(void 0!==s)return!0}}return!1},x.prototype.hasBase=function(e){return void 0===this.unit.base?void 0===e:this.unit.base===e},x.prototype.equalBase=function(e){return this.unit.base===e.unit.base},x.prototype.equals=function(e){return this.equalBase(e)&&this.value==e.value},x.prototype.toString=function(){var e;if(this.fixPrefix)return e=this._unnormalize(this.value),it.formatNumber(e)+" "+this.prefix.name+this.unit.name;var n=Math.abs(this.value/this.unit.value),t=x.PREFIX_NONE,r=Math.abs(Math.log(n/t.value)/Math.LN10-1.2),a=this.unit.prefixes;for(var i in a)if(a.hasOwnProperty(i)){var o=a[i];if(o.scientific){var s=Math.abs(Math.log(n/o.value)/Math.LN10-1.2);r>s&&(t=o,r=s)}}return e=this._unnormalize(this.value,t.value),it.formatNumber(e)+" "+t.name+this.unit.name},x.PREFIXES={NONE:{"":{name:"",value:1,scientific:!0}},SHORT:{"":{name:"",value:1,scientific:!0},da:{name:"da",value:10,scientific:!1},h:{name:"h",value:100,scientific:!1},k:{name:"k",value:1e3,scientific:!0},M:{name:"M",value:1e6,scientific:!0},G:{name:"G",value:1e9,scientific:!0},T:{name:"T",value:1e12,scientific:!0},P:{name:"P",value:1e15,scientific:!0},E:{name:"E",value:1e18,scientific:!0},Z:{name:"Z",value:1e21,scientific:!0},Y:{name:"Y",value:1e24,scientific:!0},d:{name:"d",value:.1,scientific:!1},c:{name:"c",value:.01,scientific:!1},m:{name:"m",value:.001,scientific:!0},u:{name:"u",value:1e-6,scientific:!0},n:{name:"n",value:1e-9,scientific:!0},p:{name:"p",value:1e-12,scientific:!0},f:{name:"f",value:1e-15,scientific:!0},a:{name:"a",value:1e-18,scientific:!0},z:{name:"z",value:1e-21,scientific:!0},y:{name:"y",value:1e-24,scientific:!0}},LONG:{"":{name:"",value:1,scientific:!0},deca:{name:"deca",value:10,scientific:!1},hecto:{name:"hecto",value:100,scientific:!1},kilo:{name:"kilo",value:1e3,scientific:!0},mega:{name:"mega",value:1e6,scientific:!0},giga:{name:"giga",value:1e9,scientific:!0},tera:{name:"tera",value:1e12,scientific:!0},peta:{name:"peta",value:1e15,scientific:!0},exa:{name:"exa",value:1e18,scientific:!0},zetta:{name:"zetta",value:1e21,scientific:!0},yotta:{name:"yotta",value:1e24,scientific:!0},deci:{name:"deci",value:.1,scientific:!1},centi:{name:"centi",value:.01,scientific:!1},milli:{name:"milli",value:.001,scientific:!0},micro:{name:"micro",value:1e-6,scientific:!0},nano:{name:"nano",value:1e-9,scientific:!0},pico:{name:"pico",value:1e-12,scientific:!0},femto:{name:"femto",value:1e-15,scientific:!0},atto:{name:"atto",value:1e-18,scientific:!0},zepto:{name:"zepto",value:1e-21,scientific:!0},yocto:{name:"yocto",value:1e-24,scientific:!0}},BINARY_SHORT:{"":{name:"",value:1,scientific:!0},k:{name:"k",value:1024,scientific:!0},M:{name:"M",value:Math.pow(1024,2),scientific:!0},G:{name:"G",value:Math.pow(1024,3),scientific:!0},T:{name:"T",value:Math.pow(1024,4),scientific:!0},P:{name:"P",value:Math.pow(1024,5),scientific:!0},E:{name:"E",value:Math.pow(1024,6),scientific:!0},Z:{name:"Z",value:Math.pow(1024,7),scientific:!0},Y:{name:"Y",value:Math.pow(1024,8),scientific:!0},Ki:{name:"Ki",value:1024,scientific:!0},Mi:{name:"Mi",value:Math.pow(1024,2),scientific:!0},Gi:{name:"Gi",value:Math.pow(1024,3),scientific:!0},Ti:{name:"Ti",value:Math.pow(1024,4),scientific:!0},Pi:{name:"Pi",value:Math.pow(1024,5),scientific:!0},Ei:{name:"Ei",value:Math.pow(1024,6),scientific:!0},Zi:{name:"Zi",value:Math.pow(1024,7),scientific:!0},Yi:{name:"Yi",value:Math.pow(1024,8),scientific:!0}},BINARY_LONG:{"":{name:"",value:1,scientific:!0},kilo:{name:"kilo",value:1024,scientific:!0},mega:{name:"mega",value:Math.pow(1024,2),scientific:!0},giga:{name:"giga",value:Math.pow(1024,3),scientific:!0},tera:{name:"tera",value:Math.pow(1024,4),scientific:!0},peta:{name:"peta",value:Math.pow(1024,5),scientific:!0},exa:{name:"exa",value:Math.pow(1024,6),scientific:!0},zetta:{name:"zetta",value:Math.pow(1024,7),scientific:!0},yotta:{name:"yotta",value:Math.pow(1024,8),scientific:!0},kibi:{name:"kibi",value:1024,scientific:!0},mebi:{name:"mebi",value:Math.pow(1024,2),scientific:!0},gibi:{name:"gibi",value:Math.pow(1024,3),scientific:!0},tebi:{name:"tebi",value:Math.pow(1024,4),scientific:!0},pebi:{name:"pebi",value:Math.pow(1024,5),scientific:!0},exi:{name:"exi",value:Math.pow(1024,6),scientific:!0},zebi:{name:"zebi",value:Math.pow(1024,7),scientific:!0},yobi:{name:"yobi",value:Math.pow(1024,8),scientific:!0}}},x.PREFIX_NONE={name:"",value:1,scientific:!0},x.BASE_UNITS={NONE:{},LENGTH:{},MASS:{},TIME:{},CURRENT:{},TEMPERATURE:{},LUMINOUS_INTENSITY:{},AMOUNT_OF_SUBSTANCE:{},FORCE:{},SURFACE:{},VOLUME:{},ANGLE:{},BIT:{}};var ot=x.BASE_UNITS,st=x.PREFIXES;x.BASE_UNIT_NONE={},x.UNIT_NONE={name:"",base:x.BASE_UNIT_NONE,value:1,offset:0},x.UNITS=[{name:"meter",base:ot.LENGTH,prefixes:st.LONG,value:1,offset:0},{name:"inch",base:ot.LENGTH,prefixes:st.NONE,value:.0254,offset:0},{name:"foot",base:ot.LENGTH,prefixes:st.NONE,value:.3048,offset:0},{name:"yard",base:ot.LENGTH,prefixes:st.NONE,value:.9144,offset:0},{name:"mile",base:ot.LENGTH,prefixes:st.NONE,value:1609.344,offset:0},{name:"link",base:ot.LENGTH,prefixes:st.NONE,value:.201168,offset:0},{name:"rod",base:ot.LENGTH,prefixes:st.NONE,value:5.02921,offset:0},{name:"chain",base:ot.LENGTH,prefixes:st.NONE,value:20.1168,offset:0},{name:"angstrom",base:ot.LENGTH,prefixes:st.NONE,value:1e-10,offset:0},{name:"m",base:ot.LENGTH,prefixes:st.SHORT,value:1,offset:0},{name:"ft",base:ot.LENGTH,prefixes:st.NONE,value:.3048,offset:0},{name:"yd",base:ot.LENGTH,prefixes:st.NONE,value:.9144,offset:0},{name:"mi",base:ot.LENGTH,prefixes:st.NONE,value:1609.344,offset:0},{name:"li",base:ot.LENGTH,prefixes:st.NONE,value:.201168,offset:0},{name:"rd",base:ot.LENGTH,prefixes:st.NONE,value:5.02921,offset:0},{name:"ch",base:ot.LENGTH,prefixes:st.NONE,value:20.1168,offset:0},{name:"mil",base:ot.LENGTH,prefixes:st.NONE,value:254e-7,offset:0},{name:"m2",base:ot.SURFACE,prefixes:st.SHORT,value:1,offset:0},{name:"sqin",base:ot.SURFACE,prefixes:st.NONE,value:64516e-8,offset:0},{name:"sqft",base:ot.SURFACE,prefixes:st.NONE,value:.09290304,offset:0},{name:"sqyd",base:ot.SURFACE,prefixes:st.NONE,value:.83612736,offset:0},{name:"sqmi",base:ot.SURFACE,prefixes:st.NONE,value:2589988.110336,offset:0},{name:"sqrd",base:ot.SURFACE,prefixes:st.NONE,value:25.29295,offset:0},{name:"sqch",base:ot.SURFACE,prefixes:st.NONE,value:404.6873,offset:0},{name:"sqmil",base:ot.SURFACE,prefixes:st.NONE,value:6.4516e-10,offset:0},{name:"m3",base:ot.VOLUME,prefixes:st.SHORT,value:1,offset:0},{name:"L",base:ot.VOLUME,prefixes:st.SHORT,value:.001,offset:0},{name:"litre",base:ot.VOLUME,prefixes:st.LONG,value:.001,offset:0},{name:"cuin",base:ot.VOLUME,prefixes:st.NONE,value:16387064e-12,offset:0},{name:"cuft",base:ot.VOLUME,prefixes:st.NONE,value:.028316846592,offset:0},{name:"cuyd",base:ot.VOLUME,prefixes:st.NONE,value:.764554857984,offset:0},{name:"teaspoon",base:ot.VOLUME,prefixes:st.NONE,value:5e-6,offset:0},{name:"tablespoon",base:ot.VOLUME,prefixes:st.NONE,value:15e-6,offset:0},{name:"minim",base:ot.VOLUME,prefixes:st.NONE,value:6.161152e-8,offset:0},{name:"fluiddram",base:ot.VOLUME,prefixes:st.NONE,value:36966911e-13,offset:0},{name:"fluidounce",base:ot.VOLUME,prefixes:st.NONE,value:2957353e-11,offset:0},{name:"gill",base:ot.VOLUME,prefixes:st.NONE,value:.0001182941,offset:0},{name:"cup",base:ot.VOLUME,prefixes:st.NONE,value:.0002365882,offset:0},{name:"pint",base:ot.VOLUME,prefixes:st.NONE,value:.0004731765,offset:0},{name:"quart",base:ot.VOLUME,prefixes:st.NONE,value:.0009463529,offset:0},{name:"gallon",base:ot.VOLUME,prefixes:st.NONE,value:.003785412,offset:0},{name:"beerbarrel",base:ot.VOLUME,prefixes:st.NONE,value:.1173478,offset:0},{name:"oilbarrel",base:ot.VOLUME,prefixes:st.NONE,value:.1589873,offset:0},{name:"hogshead",base:ot.VOLUME,prefixes:st.NONE,value:.238481,offset:0},{name:"fldr",base:ot.VOLUME,prefixes:st.NONE,value:36966911e-13,offset:0},{name:"floz",base:ot.VOLUME,prefixes:st.NONE,value:2957353e-11,offset:0},{name:"gi",base:ot.VOLUME,prefixes:st.NONE,value:.0001182941,offset:0},{name:"cp",base:ot.VOLUME,prefixes:st.NONE,value:.0002365882,offset:0},{name:"pt",base:ot.VOLUME,prefixes:st.NONE,value:.0004731765,offset:0},{name:"qt",base:ot.VOLUME,prefixes:st.NONE,value:.0009463529,offset:0},{name:"gal",base:ot.VOLUME,prefixes:st.NONE,value:.003785412,offset:0},{name:"bbl",base:ot.VOLUME,prefixes:st.NONE,value:.1173478,offset:0},{name:"obl",base:ot.VOLUME,prefixes:st.NONE,value:.1589873,offset:0},{name:"g",base:ot.MASS,prefixes:st.SHORT,value:.001,offset:0},{name:"gram",base:ot.MASS,prefixes:st.LONG,value:.001,offset:0},{name:"ton",base:ot.MASS,prefixes:st.SHORT,value:907.18474,offset:0},{name:"tonne",base:ot.MASS,prefixes:st.SHORT,value:1e3,offset:0},{name:"grain",base:ot.MASS,prefixes:st.NONE,value:6479891e-11,offset:0},{name:"dram",base:ot.MASS,prefixes:st.NONE,value:.0017718451953125,offset:0},{name:"ounce",base:ot.MASS,prefixes:st.NONE,value:.028349523125,offset:0},{name:"poundmass",base:ot.MASS,prefixes:st.NONE,value:.45359237,offset:0},{name:"hundredweight",base:ot.MASS,prefixes:st.NONE,value:45.359237,offset:0},{name:"stick",base:ot.MASS,prefixes:st.NONE,value:.115,offset:0},{name:"gr",base:ot.MASS,prefixes:st.NONE,value:6479891e-11,offset:0},{name:"dr",base:ot.MASS,prefixes:st.NONE,value:.0017718451953125,offset:0},{name:"oz",base:ot.MASS,prefixes:st.NONE,value:.028349523125,offset:0},{name:"lbm",base:ot.MASS,prefixes:st.NONE,value:.45359237,offset:0},{name:"cwt",base:ot.MASS,prefixes:st.NONE,value:45.359237,offset:0},{name:"s",base:ot.TIME,prefixes:st.SHORT,value:1,offset:0},{name:"min",base:ot.TIME,prefixes:st.NONE,value:60,offset:0},{name:"h",base:ot.TIME,prefixes:st.NONE,value:3600,offset:0},{name:"seconds",base:ot.TIME,prefixes:st.LONG,value:1,offset:0},{name:"second",base:ot.TIME,prefixes:st.LONG,value:1,offset:0},{name:"sec",base:ot.TIME,prefixes:st.LONG,value:1,offset:0},{name:"minutes",base:ot.TIME,prefixes:st.NONE,value:60,offset:0},{name:"minute",base:ot.TIME,prefixes:st.NONE,value:60,offset:0},{name:"hours",base:ot.TIME,prefixes:st.NONE,value:3600,offset:0},{name:"hour",base:ot.TIME,prefixes:st.NONE,value:3600,offset:0},{name:"day",base:ot.TIME,prefixes:st.NONE,value:86400,offset:0},{name:"days",base:ot.TIME,prefixes:st.NONE,value:86400,offset:0},{name:"rad",base:ot.ANGLE,prefixes:st.NONE,value:1,offset:0},{name:"deg",base:ot.ANGLE,prefixes:st.NONE,value:.017453292519943295,offset:0},{name:"grad",base:ot.ANGLE,prefixes:st.NONE,value:.015707963267948967,offset:0},{name:"cycle",base:ot.ANGLE,prefixes:st.NONE,value:6.283185307179586,offset:0},{name:"A",base:ot.CURRENT,prefixes:st.SHORT,value:1,offset:0},{name:"ampere",base:ot.CURRENT,prefixes:st.LONG,value:1,offset:0},{name:"K",base:ot.TEMPERATURE,prefixes:st.NONE,value:1,offset:0},{name:"degC",base:ot.TEMPERATURE,prefixes:st.NONE,value:1,offset:273.15},{name:"degF",base:ot.TEMPERATURE,prefixes:st.NONE,value:1/1.8,offset:459.67},{name:"degR",base:ot.TEMPERATURE,prefixes:st.NONE,value:1/1.8,offset:0},{name:"kelvin",base:ot.TEMPERATURE,prefixes:st.NONE,value:1,offset:0},{name:"celsius",base:ot.TEMPERATURE,prefixes:st.NONE,value:1,offset:273.15},{name:"fahrenheit",base:ot.TEMPERATURE,prefixes:st.NONE,value:1/1.8,offset:459.67},{name:"rankine",base:ot.TEMPERATURE,prefixes:st.NONE,value:1/1.8,offset:0},{name:"mol",base:ot.AMOUNT_OF_SUBSTANCE,prefixes:st.NONE,value:1,offset:0},{name:"mole",base:ot.AMOUNT_OF_SUBSTANCE,prefixes:st.NONE,value:1,offset:0},{name:"cd",base:ot.LUMINOUS_INTENSITY,prefixes:st.NONE,value:1,offset:0},{name:"candela",base:ot.LUMINOUS_INTENSITY,prefixes:st.NONE,value:1,offset:0},{name:"N",base:ot.FORCE,prefixes:st.SHORT,value:1,offset:0},{name:"newton",base:ot.FORCE,prefixes:st.LONG,value:1,offset:0},{name:"lbf",base:ot.FORCE,prefixes:st.NONE,value:4.4482216152605,offset:0},{name:"poundforce",base:ot.FORCE,prefixes:st.NONE,value:4.4482216152605,offset:0},{name:"b",base:ot.BIT,prefixes:st.BINARY_SHORT,value:1,offset:0},{name:"bits",base:ot.BIT,prefixes:st.BINARY_LONG,value:1,offset:0},{name:"B",base:ot.BIT,prefixes:st.BINARY_SHORT,value:8,offset:0},{name:"bytes",base:ot.BIT,prefixes:st.BINARY_LONG,value:8,offset:0}],at.E=Math.E,at.LN2=Math.LN2,at.LN10=Math.LN10,at.LOG2E=Math.LOG2E,at.LOG10E=Math.LOG10E,at.PI=Math.PI,at.SQRT1_2=Math.SQRT1_2,at.SQRT2=Math.SQRT2,at.I=new n(0,-1),at.pi=at.PI,at.e=at.E,at.i=at.I,at.abs=N,N.doc={name:"abs",category:"Arithmetic",syntax:["abs(x)"],description:"Compute the absolute value.",examples:["abs(3.5)","abs(-4.2)"],seealso:["sign"]},at.add=b,b.doc={name:"add",category:"Operators",syntax:["x + y","add(x, y)"],description:"Add two values.",examples:["2.1 + 3.6","ans - 3.6","3 + 2i",'"hello" + " world"',"3 cm + 2 inch"],seealso:["subtract"]},at.ceil=O,O.doc={name:"ceil",category:"Arithmetic",syntax:["ceil(x)"],description:"Round a value towards plus infinity.If x is complex, both real and imaginary part are rounded towards plus infinity.",examples:["ceil(3.2)","ceil(3.8)","ceil(-4.2)"],seealso:["floor","fix","round"]},at.cube=M,M.doc={name:"cube",category:"Arithmetic",syntax:["cube(x)"],description:"Compute the cube of a value. The cube of x is x * x * x.",examples:["cube(2)","2^3","2 * 2 * 2"],seealso:["multiply","square","pow"]},at.divide=A,A.doc={name:"divide",category:"Operators",syntax:["x / y","divide(x, y)"],description:"Divide two values.",examples:["2 / 3","ans * 3","4.5 / 2","3 + 4 / 2","(3 + 4) / 2","18 km / 4.5"],seealso:["multiply"]},at.equal=T,T.doc={name:"equal",category:"Operators",syntax:["x == y","equal(x, y)"],description:"Check equality of two values. Returns 1 if the values are equal, and 0 if not.",examples:["2+2 == 3","2+2 == 4","a = 3.2","b = 6-2.8","a == b","50cm == 0.5m"],seealso:["unequal","smaller","larger","smallereq","largereq"]},at.exp=z,z.doc={name:"exp",category:"Arithmetic",syntax:["exp(x)"],description:"Calculate the exponent of a value.",examples:["exp(1.3)","e ^ 1.3","log(exp(1.3))","x = 2.4","(exp(i*x) == cos(x) + i*sin(x))   # Euler's formula"],seealso:["square","multiply","log"]},at.fix=q,q.doc={name:"fix",category:"Arithmetic",syntax:["fix(x)"],description:"Round a value towards zero.If x is complex, both real and imaginary part are rounded towards zero.",examples:["fix(3.2)","fix(3.8)","fix(-4.2)","fix(-4.8)"],seealso:["ceil","floor","round"]},at.floor=U,U.doc={name:"floor",category:"Arithmetic",syntax:["floor(x)"],description:"Round a value towards minus infinity.If x is complex, both real and imaginary part are rounded towards minus infinity.",examples:["floor(3.2)","floor(3.8)","floor(-4.2)"],seealso:["ceil","fix","round"]},at.larger=R,R.doc={name:"larger",category:"Operators",syntax:["x > y","larger(x, y)"],description:"Check if value x is larger than y. Returns 1 if x is larger than y, and 0 if not.",examples:["2 > 3","5 > 2*2","a = 3.3","b = 6-2.8","(a > b)","(b < a)","5 cm > 2 inch"],seealso:["equal","unequal","smaller","smallereq","largereq"]},at.largereq=L,L.doc={name:"largereq",category:"Operators",syntax:["x >= y","largereq(x, y)"],description:"Check if value x is larger or equal to y. Returns 1 if x is larger or equal to y, and 0 if not.",examples:["2 > 1+1","2 >= 1+1","a = 3.2","b = 6-2.8","(a > b)"],seealso:["equal","unequal","smallereq","smaller","largereq"]},at.log=C,C.doc={name:"log",category:"Arithmetic",syntax:["log(x)","log(x, base)"],description:"Compute the logarithm of a value. If no base is provided, the natural logarithm of x is calculated. If base if provided, the logarithm is calculated for the specified base. log(x, base) is defined as log(x) / log(base).",examples:["log(3.5)","a = log(2.4)","exp(a)","10 ^ 3","log(1000, 10)","log(1000) / log(10)","b = logb(1024, 2)","2 ^ b"],seealso:["exp","log10"]},at.log10=_,_.doc={name:"log10",category:"Arithmetic",syntax:["log10(x)"],description:"Compute the 10-base logarithm of a value.",examples:["log10(1000)","10 ^ 3","log10(0.01)","log(1000) / log(10)","log(1000, 10)"],seealso:["exp","log"]},at.mod=I,I.doc={name:"mod",category:"Operators",syntax:["x % y","x mod y","mod(x, y)"],description:"Calculates the modulus, the remainder of an integer division.",examples:["7 % 3","11 % 2","10 mod 4","function isOdd(x) = x % 2","isOdd(2)","isOdd(3)"],seealso:[]},at.multiply=P,P.doc={name:"multiply",category:"Operators",syntax:["x * y","multiply(x, y)"],description:"multiply two values.",examples:["2.1 * 3.6","ans / 3.6","2 * 3 + 4","2 * (3 + 4)","3 * 2.1 km"],seealso:["divide"]},at.pow=k,k.doc={name:"pow",category:"Operators",syntax:["x ^ y","pow(x, y)"],description:"Calculates the power of x to y, x^y.",examples:["2^3 = 8","2*2*2","1 + e ^ (pi * i)"],seealso:["unequal","smaller","larger","smallereq","largereq"]},at.round=D,D.doc={name:"round",category:"Arithmetic",syntax:["round(x)","round(x, n)"],description:"round a value towards the nearest integer.If x is complex, both real and imaginary part are rounded towards the nearest integer. When n is specified, the value is rounded to n decimals.",examples:["round(3.2)","round(3.8)","round(-4.2)","round(-4.8)","round(pi, 3)","round(123.45678, 2)"],seealso:["ceil","floor","fix"]},at.sign=V,V.doc={name:"sign",category:"Arithmetic",syntax:["sign(x)"],description:"Compute the sign of a value. The sign of a value x is 1 when x>1, -1 when x<0, and 0 when x=0.",examples:["sign(3.5)","sign(-4.2)","sign(0)"],seealso:["abs"]},at.smaller=F,F.doc={name:"smaller",category:"Operators",syntax:["x < y","smaller(x, y)"],description:"Check if value x is smaller than value y. Returns 1 if x is smaller than y, and 0 if not.",examples:["2 < 3","5 < 2*2","a = 3.3","b = 6-2.8","(a < b)","5 cm < 2 inch"],seealso:["equal","unequal","larger","smallereq","largereq"]},at.smallereq=H,H.doc={name:"smallereq",category:"Operators",syntax:["x <= y","smallereq(x, y)"],description:"Check if value x is smaller or equal to value y. Returns 1 if x is smaller than y, and 0 if not.",examples:["2 < 1+1","2 <= 1+1","a = 3.2","b = 6-2.8","(a < b)"],seealso:["equal","unequal","larger","smaller","largereq"]},at.sqrt=Y,Y.doc={name:"sqrt",category:"Arithmetic",syntax:["sqrt(x)"],description:"Compute the square root value. If x = y * y, then y is the square root of x.",examples:["sqrt(25)","5 * 5","sqrt(-1)"],seealso:["square","multiply"]},at.square=W,W.doc={name:"square",category:"Arithmetic",syntax:["square(x)"],description:"Compute the square of a value. The square of x is x * x.",examples:["square(3)","sqrt(9)","3^2","3 * 3"],seealso:["multiply","pow","sqrt","cube"]},at.subtract=X,X.doc={name:"subtract",category:"Operators",syntax:["x - y","subtract(x, y)"],description:"subtract two values.",examples:["5.3 - 2","ans + 2","2/3 - 1/6","2 * 3 - 3","2.1 km - 500m"],seealso:["add"]},at.unaryminus=Z,Z.doc={name:"unaryminus",category:"Operators",syntax:["-x","unaryminus(x)"],description:"Inverse the sign of a value.",examples:["-4.5","-(-5.6)"],seealso:["add","subtract"]},at.unequal=K,K.doc={name:"unequal",category:"Operators",syntax:["x != y","unequal(x, y)"],description:"Check unequality of two values. Returns 1 if the values are unequal, and 0 if they are equal.",examples:["2+2 != 3","2+2 != 4","a = 3.2","b = 6-2.8","a != b","50cm != 0.5m","5 cm != 2 inch"],seealso:["equal","smaller","larger","smallereq","largereq"]},at.arg=Q,Q.doc={name:"arg",category:"Complex",syntax:["arg(x)"],description:"Compute the argument of a complex value. If x = a+bi, the argument is computed as atan2(b, a).",examples:["arg(2 + 2i)","atan2(3, 2)","arg(2 - 3i)"],seealso:["re","im","conj","abs"]},at.conj=J,J.doc={name:"conj",category:"Complex",syntax:["conj(x)"],description:"Compute the complex conjugate of a complex value. If x = a+bi, the complex conjugate is a-bi.",examples:["conj(2 + 3i)","conj(2 - 3i)","conj(-5.2i)"],seealso:["re","im","abs","arg"]},at.im=$,$.doc={name:"im",category:"Complex",syntax:["im(x)"],description:"Get the imaginary part of a complex number.",examples:["im(2 + 3i)","re(2 + 3i)","im(-5.2i)","im(2.4)"],seealso:["re","conj","abs","arg"]},at.re=en,en.doc={name:"re",category:"Complex",syntax:["re(x)"],description:"Get the real part of a complex number.",examples:["re(2 + 3i)","im(2 + 3i)","re(-5.2i)","re(2.4)"],seealso:["im","conj","abs","arg"]},at.complex=nn,at.matrix=tn,at.parser=rn,at.range=an,at.unit=on,at.workspace=sn,at.concat=fn,fn.doc={name:"concat",category:"Numerics",syntax:["concat(a, b, c, ...)","concat(a, b, c, ..., dim)"],description:"Concatenate matrices. By default, the matrices are concatenated by the first dimension. The dimension on which to concatenate can be provided as last argument.",examples:["a = [1, 2; 5, 6]","b = [3, 4; 7, 8]","concat(a, b)","[a, b]","concat(a, b, 2)","[a; b]"],seealso:[]},at.det=cn,cn.doc={name:"det",category:"Numerics",syntax:["det(x)"],description:"Calculate the determinant of a matrix",examples:["det([1, 2; 3, 4])","det([-2, 2, 3; -1, 1, 3; 2, 0, -1])"],seealso:["concat","diag","eye","inv","range","size","squeeze","transpose","zeros"]},at.diag=mn,mn.doc={name:"diag",category:"Matrix",syntax:["diag(x)","diag(x, k)"],description:"Create a diagonal matrix or retrieve the diagonal of a matrix. When x is a vector, a matrix with the vector values on the diagonal will be returned. When x is a matrix, a vector with the diagonal values of the matrix is returned.When k is provided, the k-th diagonal will be filled in or retrieved, if k is positive, the values are placed on the super diagonal. When k is negative, the values are placed on the sub diagonal.",examples:["diag(1:4)","diag(1:4, 1)","a = [1, 2, 3; 4, 5, 6; 7, 8, 9]","diag(a)"],seealso:["concat","det","eye","inv","ones","range","size","squeeze","transpose","zeros"]},at.eye=pn,pn.doc={name:"eye",category:"Numerics",syntax:["eye(n)","eye(m, n)","eye([m, n])","eye"],description:"Returns the identity matrix with size m-by-n. The matrix has ones on the diagonal and zeros elsewhere.",examples:["eye(3)","eye(3, 5)","a = [1, 2, 3; 4, 5, 6]","eye(size(a))"],seealso:["concat","det","diag","inv","ones","range","size","squeeze","transpose","zeros"]},at.inv=vn,vn.doc={name:"inv",category:"Numerics",syntax:["inv(x)"],description:"Calculate the inverse of a matrix",examples:["inv([1, 2; 3, 4])","inv(4)","1 / 4"],seealso:["concat","det","diag","eye","ones","range","size","squeeze","transpose","zeros"]},at.ones=gn,gn.doc={name:"ones",category:"Numerics",syntax:["ones(n)","ones(m, n)","ones(m, n, p, ...)","ones([m, n])","ones([m, n, p, ...])","ones"],description:"Create a matrix containing ones.",examples:["ones(3)","ones(3, 5)","ones([2,3]) * 4.5","a = [1, 2, 3; 4, 5, 6]","ones(size(a))"],seealso:["concat","det","diag","eye","inv","range","size","squeeze","transpose","zeros"]},at.size=yn,yn.doc={name:"size",category:"Numerics",syntax:["size(x)"],description:"Calculate the size of a matrix.",examples:["size(2.3)",'size("hello world")',"a = [1, 2; 3, 4; 5, 6]","size(a)","size(1:6)"],seealso:["concat","det","diag","eye","inv","ones","range","squeeze","transpose","zeros"]},at.squeeze=xn,xn.doc={name:"squeeze",category:"Numerics",syntax:["squeeze(x)"],description:"Remove singleton dimensions from a matrix.",examples:["a = zeros(1,3,2)","size(squeeze(a))","b = zeros(3,1,1)","size(squeeze(b))"],seealso:["concat","det","diag","eye","inv","ones","range","size","transpose","zeros"]},at.transpose=En,En.doc={name:"transpose",category:"Numerics",syntax:["transpose(x)"],description:"Transpose a matrix",examples:["a = [1, 2, 3; 4, 5, 6]","transpose(a)"],seealso:["concat","det","diag","eye","inv","ones","range","size","squeeze","zeros"]},at.zeros=Nn,Nn.doc={name:"zeros",category:"Numerics",syntax:["zeros(n)","zeros(m, n)","zeros(m, n, p, ...)","zeros([m, n])","zeros([m, n, p, ...])","zeros"],description:"Create a matrix containing zeros.",examples:["zeros(3)","zeros(3, 5)","a = [1, 2, 3; 4, 5, 6]","zeros(size(a))"],seealso:["concat","det","diag","eye","inv","ones","range","size","squeeze","transpose"]},at.factorial=bn,bn.doc={name:"factorial",category:"Probability",syntax:["x!","factorial(x)"],description:"Compute the factorial of a value",examples:["5!","5*4*3*2*1","3!"],seealso:[]},at.random=On,On.doc={name:"random",category:"Probability",syntax:["random()"],description:"Return a random number between 0 and 1.",examples:["random()","100 * random()"],seealso:[]},at.max=Mn,Mn.doc={name:"max",category:"Statistics",syntax:["max(a, b, c, ...)"],description:"Compute the maximum value of a list of values.",examples:["max(2, 3, 4, 1)","max(2.7, 7.1, -4.5, 2.0, 4.1)","min(2.7, 7.1, -4.5, 2.0, 4.1)"],seealso:["sum","prod","avg","var","std","min","median"]},at.min=Tn,Tn.doc={name:"min",category:"Statistics",syntax:["min(a, b, c, ...)"],description:"Compute the minimum value of a list of values.",examples:["min(2, 3, 4, 1)","min(2.7, 7.1, -4.5, 2.0, 4.1)","max(2.7, 7.1, -4.5, 2.0, 4.1)"],seealso:["sum","prod","avg","var","std","min","median"]},at.acos=Un,Un.doc={name:"acos",category:"Trigonometry",syntax:["acos(x)"],description:"Compute the inverse cosine of a value in radians.",examples:["acos(0.5)","acos(cos(2.3))"],seealso:["cos","acos","asin"]},at.asin=Rn,Rn.doc={name:"asin",category:"Trigonometry",syntax:["asin(x)"],description:"Compute the inverse sine of a value in radians.",examples:["asin(0.5)","asin(sin(2.3))"],seealso:["sin","acos","asin"]},at.atan=Ln,Ln.doc={name:"atan",category:"Trigonometry",syntax:["atan(x)"],description:"Compute the inverse tangent of a value in radians.",examples:["atan(0.5)","atan(tan(2.3))"],seealso:["tan","acos","asin"]},at.atan2=Cn,Cn.doc={name:"atan2",category:"Trigonometry",syntax:["atan2(y, x)"],description:"Computes the principal value of the arc tangent of y/x in radians.",examples:["atan2(2, 2) / pi","angle = 60 deg in rad","x = cos(angle)","y = sin(angle)","atan2(y, x)"],seealso:["sin","cos","tan"]},at.cos=_n,_n.doc={name:"cos",category:"Trigonometry",syntax:["cos(x)"],description:"Compute the cosine of x in radians.",examples:["cos(2)","cos(pi / 4) ^ 2","cos(180 deg)","cos(60 deg)","sin(0.2)^2 + cos(0.2)^2"],seealso:["acos","sin","tan"]},at.cot=In,In.doc={name:"cot",category:"Trigonometry",syntax:["cot(x)"],description:"Compute the cotangent of x in radians. Defined as 1/tan(x)",examples:["cot(2)","1 / tan(2)"],seealso:["sec","csc","tan"]},at.csc=Pn,Pn.doc={name:"csc",category:"Trigonometry",syntax:["csc(x)"],description:"Compute the cosecant of x in radians. Defined as 1/sin(x)",examples:["csc(2)","1 / sin(2)"],seealso:["sec","cot","sin"]},at.sec=Bn,Bn.doc={name:"sec",category:"Trigonometry",syntax:["sec(x)"],description:"Compute the secant of x in radians. Defined as 1/cos(x)",examples:["sec(2)","1 / cos(2)"],seealso:["cot","csc","cos"]},at.sin=kn,kn.doc={name:"sin",category:"Trigonometry",syntax:["sin(x)"],description:"Compute the sine of x in radians.",examples:["sin(2)","sin(pi / 4) ^ 2","sin(90 deg)","sin(30 deg)","sin(0.2)^2 + cos(0.2)^2"],seealso:["asin","cos","tan"]},at.tan=Gn,Gn.doc={name:"tan",category:"Trigonometry",syntax:["tan(x)"],description:"Compute the tangent of x in radians.",examples:["tan(0.5)","sin(0.5) / cos(0.5)","tan(pi / 4)","tan(45 deg)"],seealso:["atan","sin","cos"]},at.in=Dn,Dn.doc={name:"in",category:"Units",syntax:["x in unit","in(x, unit)"],description:"Change the unit of a value.",examples:["5 inch in cm","3.2kg in g","16 bytes in bits"],seealso:[]},at.clone=jn,jn.doc={name:"clone",category:"Utils",syntax:["clone(x)"],description:"Clone a variable. Creates a copy of primitive variables,and a deep copy of matrices",examples:["clone(3.5)","clone(2 - 4i)","clone(45 deg)","clone([1, 2; 3, 4])",'clone("hello world")'],seealso:[]},at.format=Vn,Vn.doc={name:"format",category:"Utils",syntax:["format(value)"],description:"Format a value of any type as string.",examples:["format(2.3)","format(3 - 4i)","format([])"],seealso:[]},at.help=Fn,Fn.doc={name:"help",category:"Utils",syntax:["help(object)"],description:"Display documentation on a function or data type.",examples:['help("sqrt")','help("Complex")'],seealso:[]},at["import"]=Yn,Yn.doc={name:"import",category:"Utils",syntax:["import(string)"],description:"Import functions from a file.",examples:['import("numbers")','import("./mylib.js")'],seealso:[]},at["typeof"]=Xn,Xn.doc={name:"typeof",category:"Utils",syntax:["typeof(x)"],description:"Get the type of a variable.",examples:["typeof(3.5)","typeof(2 - 4i)","typeof(45 deg)",'typeof("hello world")'],seealso:[]},at.expr.node.Node=Zn,Zn.prototype.eval=function(){throw Error("Cannot evaluate a Node interface")
},Zn.prototype.toString=function(){return""},Kn.prototype=new Zn,at.expr.node.Symbol=Kn,Kn.prototype.hasParams=function(){return void 0!=this.params&&this.params.length>0},Kn.prototype.eval=function(){var e=this.fn;if(void 0===e)throw Error("Undefined symbol "+this.name);var n=this.params.map(function(e){return e.eval()});return e.apply(this,n)},Kn.prototype.toString=function(){if(this.name&&!this.params)return this.name;var e=this.name;return this.params&&this.params.length&&(e+="("+this.params.join(", ")+")"),e},Qn.prototype=new Zn,at.expr.node.Constant=Qn,Qn.prototype.eval=function(){return this.value},Qn.prototype.toString=function(){return this.value?at.format(this.value):""},Jn.prototype=new Zn,at.expr.node.MatrixNode=Jn,function(){function e(n){return n.map(function(n){return n instanceof Array?e(n):n.eval()})}function n(e){for(var n=[],r=e.length,a=0;r>a;a++){for(var i=e[a],o=i.length,s=null,f=null,u=0;o>u;u++){var c,l=at.clone(i[u]);if(l instanceof t){if(c=l.size(),l=l.valueOf(),1==c.length)l=[l],c=[1,c[0]];else if(c.length>2)throw Error("Cannot merge a multi dimensional matrix")}else l instanceof g?(l=[l.valueOf()],c=[1,l[0].length]):(c=[1,1],l=[[l]]);if(null==s)s=l,f=c[0];else{if(c[0]!=f)throw Error("Dimension mismatch ("+c[0]+" != "+f+")");for(var h=0;f>h;h++)s[h]=s[h].concat(l[h])}}n=n.concat(s)}return n}function r(e){return e.some(function(e){return e instanceof t||e instanceof g?!0:e instanceof Array?r(e):!1})}Jn.prototype.eval=function(){var a=e(this.nodes);return r(a)&&(a=n(a)),new t(a)},Jn.prototype.toString=function(){return it.formatArray(this.nodes)}}(),$n.prototype=new Zn,at.expr.node.Block=$n,$n.prototype.add=function(e,n){var t=this.params.length;this.params[t]=e,this.visible[t]=void 0!=n?n:!0},$n.prototype.eval=function(){for(var e=[],n=0,t=this.params.length;t>n;n++){var r=this.params[n].eval();this.visible[n]&&e.push(r)}return e},$n.prototype.toString=function(){for(var e=[],n=0,t=this.params.length;t>n;n++)this.visible[n]&&e.push("\n  "+(""+this.params[n]));return"["+e.join(",")+"\n]"},et.prototype=new Zn,at.expr.node.Assignment=et,et.prototype.eval=function(){if(void 0===this.expr)throw Error("Undefined symbol "+this.name);var e,n=this.params;if(n&&n.length){var t=[];this.params.forEach(function(e){t.push(e.eval())});var r=this.expr.eval();if(void 0==this.result.value)throw Error("Undefined symbol "+this.name);var a=this.result();if(!a.set)throw new TypeError("Cannot apply a subset to object of type "+at.typeof(a));e=a.set(t,r),this.result.value=e}else e=this.expr.eval(),this.result.value=e;return e},et.prototype.toString=function(){var e="";return e+=this.name,this.params&&this.params.length&&(e+="("+this.params.join(", ")+")"),e+=" = ",e+=""+this.expr},nt.prototype=new Zn,at.expr.node.Arguments=nt,nt.prototype.eval=function(){var e=this.object;if(void 0==e)throw Error("Node undefined");for(var n=e.eval(),t=this.params,r=[],a=0,i=t.length;i>a;a++)r[a]=t[a].eval();if(!n.get)throw new TypeError("Cannot apply arguments to object of type "+at.typeof(n));return n.get(r)},nt.prototype.toString=function(){var e=this.object?""+this.object:"";return this.params&&(e+="("+this.params.join(", ")+")"),e},tt.prototype=new Zn,at.expr.node.FunctionAssignment=tt,tt.prototype.createFunction=function(e,n,t,r){var a=function(){var n=t?t.length:0,a=arguments?arguments.length:0;if(n!=a)throw E(e,a,n);if(n>0)for(var i=0;n>i;i++)t[i].value=arguments[i];return r.eval()};return a.toString=function(){return e+"("+n.join(", ")+")"},a},tt.prototype.eval=function(){for(var e=this.variables,n=this.values,t=0,r=e.length;r>t;t++)e[t].value=n[t];return this.result.value=this.def,this.def},tt.prototype.toString=function(){return""+this.def},function(){function e(e){this.parentScope=e,this.nestedScopes=void 0,this.symbols={},this.defs={},this.updates={},this.links={}}at.expr.Scope=e,e.prototype.createNestedScope=function(){var n=new e(this);return this.nestedScopes||(this.nestedScopes=[]),this.nestedScopes.push(n),n},e.prototype.clear=function(){if(this.symbols={},this.defs={},this.links={},this.updates={},this.nestedScopes)for(var e=this.nestedScopes,n=0,t=e.length;t>n;n++)e[n].clear()},e.prototype.createSymbol=function(e){var n=this.symbols[e];if(!n){var t=this.findDef(e);n=this.newSymbol(e,t),this.symbols[e]=n}return n},e.prototype.newSymbol=function(e,n){var r=this,a=function(){var n,i;if(!a.value&&(a.value=r.findDef(e),!a.value))throw Error("Undefined symbol "+e);if("function"==typeof a.value)return a.value.apply(null,arguments);if(a.value instanceof t||a.value instanceof g||a.value instanceof Array){if(arguments.length){var o=a.value instanceof Array?new t(a.value):a.value;for(n=[],i=0;arguments.length>i;i++)n[i]=arguments[i];return o.get(n)}return a.value}return a.value};return a.value=n,a.toString=function(){return a.value?""+a.value:""},a},e.prototype.createLink=function(e){var n=this.links[e];return n||(n=this.createSymbol(e),this.links[e]=n),n},e.prototype.createDef=function(e,n){var t=this.defs[e];return t||(t=this.createSymbol(e),this.defs[e]=t),t&&void 0!=n&&(t.value=n),t},e.prototype.createUpdate=function(e){var n=this.updates[e];return n||(n=this.createLink(e),this.updates[e]=n),n},e.prototype.findDef=function(e){function t(e,n){var t=a(e,n);return i[e]=t,o[e]=t,t}var r;if(r=this.defs[e])return r;if(r=this.updates[e])return r;if(this.parentScope)return this.parentScope.findDef(e);var a=this.newSymbol,i=this.symbols,o=this.defs;if("pi"==e)return t(e,at.PI);if("e"==e)return t(e,at.E);if("i"==e)return t(e,new n(0,1));var s=at[e];if(s)return t(e,s);if(x.isUnit(e)){var f=new x(null,e);return t(e,f)}return void 0},e.prototype.removeLink=function(e){delete this.links[e]},e.prototype.removeDef=function(e){delete this.defs[e]},e.prototype.removeUpdate=function(e){delete this.updates[e]},e.prototype.init=function(){var e=this.symbols,n=this.parentScope;for(var t in e)if(e.hasOwnProperty(t)){var r=e[t];r.value=n?n.findDef(t):void 0}this.nestedScopes&&this.nestedScopes.forEach(function(e){e.init()})},e.prototype.hasLink=function(e){if(this.links[e])return!0;if(this.nestedScopes)for(var n=this.nestedScopes,t=0,r=n.length;r>t;t++)if(n[t].hasLink(e))return!0;return!1},e.prototype.hasDef=function(e){return void 0!=this.defs[e]},e.prototype.hasUpdate=function(e){return void 0!=this.updates[e]},e.prototype.getUndefinedSymbols=function(){var e=this.symbols,n=[];for(var t in e)if(e.hasOwnProperty(t)){var r=e[t];void 0==r.value&&n.push(r)}return this.nestedScopes&&this.nestedScopes.forEach(function(e){n=n.concat(e.getUndefinedSymbols())}),n}}(),function(){function e(){j++,V=D.charAt(j)}function t(){j=0,V=D.charAt(0)}function r(){for(H=G.NULL,F="";" "==V||"	"==V;)e();if("#"==V)for(;"\n"!=V&&""!=V;)e();if(""==V)return H=G.DELIMITER,void 0;if("-"==V||","==V||"("==V||")"==V||"["==V||"]"==V||'"'==V||"\n"==V||";"==V||":"==V||"!"==V||"'"==V)return H=G.DELIMITER,F+=V,e(),void 0;if(a(V))for(H=G.DELIMITER;a(V);)F+=V,e();else if(o(V)){for(H=G.NUMBER;o(V);)F+=V,e();if("E"==V||"e"==V)for(F+=V,e(),("+"==V||"-"==V)&&(F+=V,e()),s(V)||(H=G.UNKNOWN);s(V);)F+=V,e()}else{if(!i(V)){for(H=G.UNKNOWN;""!=V;)F+=V,e();throw _('Syntax error in part "'+F+'"')}for(H=G.SYMBOL;i(V)||s(V);)F+=V,e()}}function a(e){return"&"==e||"|"==e||"<"==e||">"==e||"="==e||"+"==e||"/"==e||"*"==e||"%"==e||"^"==e||","==e||";"==e||"\n"==e||"!"==e||"'"==e}function i(e){return e>="a"&&"z">=e||e>="A"&&"Z">=e||"_"==e}function o(e){return e>="0"&&"9">=e||"."==e}function s(e){return e>="0"&&"9">=e}function f(e){t(),r();var n;if(n=""==F?new Qn(void 0):c(e),""!=F)throw H==G.DELIMITER?P("Unknown operator "+F):_('Unexpected part "'+F+'"');return n}function u(e){var n=l(e);if(!(n instanceof et)){var t="ans",r=void 0,a=e.createDef(t);return new et(t,r,n,a)}return n}function c(e){var n,t,a;for("\n"!=F&&";"!=F&&""!=F&&(n=u(e));"\n"==F||";"==F;)t||(t=new $n,n&&(a=";"!=F,t.add(n,a))),r(),"\n"!=F&&";"!=F&&""!=F&&(n=u(e),a=";"!=F,t.add(n,a));return t?t:(n||(n=u(e)),n)}function l(e){if(H==G.SYMBOL&&"function"==F){if(r(),H!=G.SYMBOL)throw _("Function name expected");var n=F;if(r(),"("!=F)throw _("Opening parenthesis ( expected");for(var t=e.createNestedScope(),a=[],i=[];;){if(r(),H!=G.SYMBOL)throw _("Variable name expected");var o=F,s=t.createDef(o);if(a.push(o),i.push(s),r(),","!=F){if(")"==F)break;throw _('Comma , or closing parenthesis ) expected"')}}if(r(),"="!=F)throw _("Equal sign = expected");r();var f=m(t),u=e.createDef(n);return new tt(n,a,i,f,u)}return h(e)}function h(e){var n=!1;H==G.SYMBOL&&(n=e.hasLink(F));var t=m(e);if("="==F){if(!(t instanceof Kn))throw _("Symbol expected at the left hand side of assignment operator =");var a=t.name,i=t.params;n||e.removeLink(a),r();var o=m(e),s=t.hasParams()?e.createUpdate(a):e.createDef(a);return new et(a,i,o,s)}return t}function m(e){var n=p(e);if(":"==F){for(var t=[n];":"==F;)r(),t.push(p(e));if(t.length>3)throw new TypeError("Invalid range");var a="range",i=an;n=new Kn(a,i,t)}return n}function p(e){for(var n=v(e),t={"in":"in"};void 0!==t[F];){var a=F,i=at[t[a]];r();var o=[n,v(e)];n=new Kn(a,i,o)}return n}function v(e){var n=d(e);return n}function d(e){for(var n=g(e),t={"==":"equal","!=":"unequal","<":"smaller",">":"larger","<=":"smallereq",">=":"largereq"};void 0!==t[F];){var a=F,i=at[t[a]];r();var o=[n,g(e)];n=new Kn(a,i,o)}return n}function g(e){for(var n=y(e),t={"+":"add","-":"subtract"};void 0!==t[F];){var a=F,i=at[t[a]];r();var o=[n,y(e)];n=new Kn(a,i,o)}return n}function y(e){for(var n=w(e),t={"*":"multiply","/":"divide","%":"mod",mod:"mod"};void 0!==t[F];){var a=F,i=at[t[a]];r();var o=[n,w(e)];n=new Kn(a,i,o)}return n}function w(e){if("-"==F){var n=F,t=Z;r();var a=[E(e)];return new Kn(n,t,a)}return E(e)}function E(e){for(var n=[N(e)];"^"==F;)r(),n.push(N(e));for(var t=n.pop();n.length;){var a=n.pop(),i="^",o=k,s=[a,t];t=new Kn(i,o,s)}return t}function N(e){for(var n=b(e);"!"==F;){var t=F,a=bn;r();var i=[n];n=new Kn(t,a,i)}return n}function b(e){for(var n=O(e);"'"==F;){var t=F,a=En;r();var i=[n];n=new Kn(t,a,i)}return n}function O(e){return M(e)}function M(e){if(H==G.SYMBOL){var n=F;r();var t=e.createLink(n),a=A(e),i=new Kn(n,t,a);return i}return S(e)}function A(e){var n=[];if("("==F){if(r(),")"!=F)for(n.push(m(e));","==F;)r(),n.push(m(e));if(")"!=F)throw _("Parenthesis ) missing");r()}return n}function S(n){if('"'==F){for(var t="",a="";""!=V&&('"'!=V||"\\"==a);)t+=V,a=V,e();if(r(),'"'!=F)throw _('End of string " missing');r();var i=new Qn(t);return i}return T(n)}function T(e){if("["==F){var n;for(r();"\n"==F;)r();if("]"!=F){var t=[],a=0,i=0;for(t[0]=[m(e)];","==F||";"==F;){for(","==F?i++:(a++,i=0,t[a]=[]),r();"\n"==F;)r();for(t[a][i]=m(e);"\n"==F;)r()}var o=t.length,s=t.length>0?t[0].length:0;for(a=1;o>a;a++)if(t[a].length!=s)throw P("Number of columns must match ("+t[a].length+" != "+s+")");if("]"!=F)throw _("End of matrix ] missing");r(),n=new Jn(t)}else r(),n=new Jn([[]]);return n}return z(e)}function z(e){if(H==G.NUMBER){var t;t="."==F?0:Number(F),r();var a;if(H==G.SYMBOL){if("i"==F||"I"==F)return a=new n(0,t),r(),new Qn(a);if(x.isUnit(F))return a=new x(t,F),r(),new Qn(a);throw I('Unknown unit "'+F+'"')}var i=new Qn(t);return i}return q(e)}function q(e){if("("==F){r();var n=m(e);if(")"!=F)throw _("Parenthesis ) expected");return r(),n}return U(e)}function U(){throw""==F?_("Unexpected end of expression"):_("Value expected")}function R(){return void 0}function L(){return j-F.length+1}function C(e){var n=R(),t=L();return void 0===n?void 0===t?e:e+" (char "+t+")":e+" (line "+n+", char "+t+")"}function _(e){return new SyntaxError(C(e))}function I(e){return new TypeError(C(e))}function P(e){return Error(C(e))}at.expr.Parser=function B(){if(this.constructor!=B)throw new SyntaxError("Parser constructor must be called with the new operator");this.scope=new at.expr.Scope},at.expr.Parser.prototype.parse=function(e,n){return D=e||"",n||(this._newScope(),n=this.scope),f(n)},at.expr.Parser.prototype.eval=function(e){var n=this.parse(e);return n.eval()},at.expr.Parser.prototype.get=function(e){this._newScope();var n=this.scope.findDef(e);return n?n.value:void 0},at.expr.Parser.prototype.set=function(e,n){this.scope.createDef(e,n)},at.expr.Parser.prototype._newScope=function(){this.scope=new at.expr.Scope(this.scope)},at.expr.Parser.prototype.clear=function(){this.scope.clear()};var G={NULL:0,DELIMITER:1,NUMBER:2,SYMBOL:3,UNKNOWN:4},D="",j=0,V="",F="",H=G.NULL}(),function(){function e(){this.idMax=-1,this.updateSeq=0,this.parser=new at.expr.Parser,this.scope=new at.expr.Scope,this.nodes={},this.firstNode=void 0,this.lastNode=void 0}at.expr.Workspace=e,e.prototype.clear=function(){this.nodes={},this.firstNode=void 0,this.lastNode=void 0},e.prototype.append=function(n){var t=this._getNewId(),r=this.lastNode?this.lastNode.scope:this.scope,a=new at.expr.Scope(r),i=new e.Node({id:t,expression:n,parser:this.parser,scope:a,nextNode:void 0,previousNode:this.lastNode});return this.nodes[t]=i,this.firstNode||(this.firstNode=i),this.lastNode&&(this.lastNode.nextNode=i),this.lastNode=i,this._update([t]),t},e.prototype.insertBefore=function(n,t){var r=this.nodes[t];if(!r)throw'Node with id "'+t+'" not found';var a=r.previousNode,i=this._getNewId(),o=a?a.scope:this.scope,s=new at.expr.Scope(o),f=new e.Node({id:i,expression:n,parser:this.parser,scope:s,nextNode:r,previousNode:a});this.nodes[i]=f,a?a.nextNode=f:this.firstNode=f,r.previousNode=f,r.scope.parentScope=f.scope;var u=this.getDependencies(i);return-1==u.indexOf(i)&&u.unshift(i),this._update(u),i},e.prototype.insertAfter=function(e,n){var t=this.nodes[n];if(!t)throw'Node with id "'+n+'" not found';return t==this.lastNode?this.append(e):this.insertBefore(n+1,e)},e.prototype.remove=function(e){var n=this.nodes[e];if(!n)throw'Node with id "'+e+'" not found';var t=this.getDependencies(e),r=n.previousNode,a=n.nextNode;r?r.nextNode=a:this.firstNode=a,a?a.previousNode=r:this.lastNode=r;var i=r?r.scope:this.scope;a&&(a.scope.parentScope=i),delete this.nodes[e],this._update(t)},e.prototype.replace=function(n,t){var r=this.nodes[t];if(!r)throw'Node with id "'+t+'" not found';var a=[t];e._merge(a,this.getDependencies(t));var i=r.previousNode;r.nextNode,i?i.scope:this.scope,r.setExpr(n),e._merge(a,this.getDependencies(t)),this._update(a)},e.Node=function(e){this.id=e.id,this.parser=e.parser,this.scope=e.scope,this.nextNode=e.nextNode,this.previousNode=e.previousNode,this.updateSeq=0,this.result=void 0,this.setExpr(e.expression)},e.Node.prototype.setExpr=function(e){this.expression=e||"",this.scope.clear(),this._parse()},e.Node.prototype.getExpr=function(){return this.expression},e.Node.prototype.getResult=function(){return this.result},e.Node.prototype._parse=function(){try{this.fn=this.parser.parse(this.expression,this.scope)}catch(e){var n="Error: "+((e.message||e)+"");this.fn=new Qn(n)}},e.Node.prototype.eval=function(){try{this.scope.init(),this.result=this.fn.eval()}catch(e){this.scope.init(),this.result="Error: "+((e.message||e)+"")}return this.result},e._merge=function(e,n){for(var t=0,r=n.length;r>t;t++){var a=n[t];-1==e.indexOf(a)&&e.push(a)}},e.prototype.getDependencies=function(n){var t,r=[],a=this.nodes[n];if(a){var i=a.scope.defs,o=a.scope.updates,s=[];for(t in i)i.hasOwnProperty(t)&&s.push(t);for(t in o)o.hasOwnProperty(t)&&-1==s.indexOf(t)&&s.push(t);for(var f=a.nextNode;f&&s.length;){for(var u=f.scope,c=0;s.length>c;){if(t=s[c],(u.hasLink(t)||u.hasUpdate(t))&&-1==r.indexOf(f.id)){r.push(f.id);var l=this.getDependencies(f.id);e._merge(r,l)}u.hasDef(t)&&(s.splice(c,1),c--),c++}f=f.nextNode}}return r},e.prototype.getExpr=function(e){var n=this.nodes[e];if(!n)throw'Node with id "'+e+'" not found';return n.getExpr()},e.prototype.getResult=function(e){var n=this.nodes[e];if(!n)throw'Node with id "'+e+'" not found';return n.getResult()},e.prototype._update=function(e){this.updateSeq++;for(var n=this.updateSeq,t=this.nodes,r=0,a=e.length;a>r;r++){var i=e[r],o=t[i];o&&(o.eval(),o.updateSeq=n)}},e.prototype.getChanges=function(e){var n=[],t=this.firstNode;for(e=e||0;t;)t.updateSeq>e&&n.push(t.id),t=t.nextNode;return{ids:n,updateSeq:this.updateSeq}},e.prototype._getNewId=function(){return this.idMax++,this.idMax},e.prototype.toString=function(){return JSON.stringify(this.toJSON())},e.prototype.toJSON=function(){for(var e=[],n=this.firstNode;n;){var t={id:n.id,expression:n.expression,dependencies:this.getDependencies(n.id)};try{t.result=n.getResult()}catch(r){t.result="Error: "+((r.message||r)+"")}e.push(t),n=n.nextNode}return e}}(),at.Complex=function(){rt("new math.Complex()","math.complex()")},at.Unit=function(){rt("new math.Unit()","math.unit()")},at.parser.Parser=function(){rt("new math.parser.Parser()","math.parser()")},at.parser.Workspace=function(){rt("new math.parser.Workspace()","math.workspace()")}})();